/* vi:set ts=2 sw=2 et:
 *
 * emelFM
 * Copyright (C) 1999 Michael Clark.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include "emelfm2.h"

// binary factor
#define BFAC 1024.0
#define BFAC2 1048576.0
#define BFAC3 1073741824.0

static void add_disk_usage(gchar *filename, guint64 *total, guint64 *files, guint64 *dirs)
{
  struct stat statbuf;

  if (lstat(filename, &statbuf) != 0)
  {
    status_errno();
    return;
  }
  (*total) += statbuf.st_size;
  if (S_ISDIR(statbuf.st_mode))
  {
    DIR *dp;
    struct dirent *entry;
    struct stat statbuf;
    gchar path[PATH_MAX];

    (*dirs)++;
    if ((dp = opendir(filename)) == NULL)
    {
      status_errno();
      status_message("Warning: couldn't open directory ");
      status_message(filename);
      status_message("\n");
      return;
    }

    while ((entry = readdir(dp)) != NULL)
    {
      if (g_str_equal(entry->d_name, ".") || g_str_equal(entry->d_name, ".."))
        continue;

      g_snprintf(path, PATH_MAX, "%s/%s", filename, entry->d_name);
      if (lstat(path, &statbuf) != 0)
      {
        status_errno();
        continue;
      }

      (*total) += statbuf.st_size;
      if (S_ISDIR(statbuf.st_mode))
        add_disk_usage(path, total, files, dirs);
      else    
        (*files)++;
    }
    closedir(dp);
  }
  else
    (*files)++;
}

static void du ()
{
  GList *base, *tmp;
  FileInfo *info;
  guint64 total = 0, files = 0, dirs = 0;
  GString *text;
  
  base = tmp = get_selection(curr_view);
  for (; tmp != NULL; tmp = tmp->next)
  {
    info = tmp->data;
    printd (DEBUG, "fn: %s", info->filename);
    add_disk_usage(g_build_filename (curr_view->dir, info->filename, NULL), &total, &files, &dirs);
  }

  text = g_string_new("total size: ");
  if (total < (10 * BFAC))
    g_string_append_printf(text, "%lld b\n", total);
  else if (total < 10 * BFAC2)
    g_string_append_printf(text, "%.2f kb\n", (total / BFAC));
  else if (total < 10 * BFAC3)
    g_string_append_printf(text, "%.2f mb\n", (total / BFAC2));
  else
    g_string_append_printf(text, "%.2f gb\n", (total / BFAC3));

  g_string_append_printf(text, "in  %lld files\nand %lld directories\n", files, dirs);
  status_message (text->str);
  status_message2 ("----end-of-output----\n", FALSE, "small", "grey", NULL);
  g_string_free(text, TRUE);
}

gint init_plugin (Plugin *p)
{
  p->name = "Disk Usage";
  p->description = "Calculates the disk usage of the selected files.";
  p->plugin_cb = du;

  return TRUE;
}


