/* $Id: e2_action.h,v 1.12 2004/08/07 22:42:50 cairo Exp $
 * 
 * Copyright (C) 2003-2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/**
 * @file src/actions/e2_action.h 
 * @brief action system header
 *
 * This is the header file for the action system.
 **/

#ifndef E2_ACTION_H
#	define E2_ACTION_H

typedef enum
{
	E2_ACTION_TYPE_ITEM,
	E2_ACTION_TYPE_SUBMENU,
	E2_ACTION_TYPE_FILE_ACTIONS,
	E2_ACTION_TYPE_CUSTOM_COMMAND,
	E2_ACTION_TYPE_PLUGINS,
	E2_ACTION_TYPE_SEPARATOR,
	E2_ACTION_TYPE_TEAR_OFF_MENU,
	E2_ACTION_TYPE_TOGGLE,
	E2_ACTION_TYPE_COMMAND_LINE,
	E2_ACTION_TYPE_BOOKMARKS,
	E2_ACTION_TYPE_DUMMY
} E2_ACTION_TYPE;

typedef enum
{
	E2_ACTION_EXCLUDE_TOOLBAR = 1 << 0,
	E2_ACTION_EXCLUDE_MENU = 1 << 1,
	E2_ACTION_EXCLUDE_ACCEL = 1 << 2,
} E2_ACTION_EXCLUDE;

typedef struct _E2_Action
{
	gchar *name;
	E2_ACTION_TYPE type;
	gpointer func;
	gboolean has_arg;
	E2_ACTION_EXCLUDE exclude;
	gpointer data;
	gpointer data2;
	GtkTreeRowReference *ref;
} E2_Action;

typedef struct _E2_ActionRuntime
{
	E2_Action *action;
	gpointer data;
	gpointer data_free;
} E2_ActionRuntime;

GtkTreeModel *e2_actions_fill_store (E2_ACTION_EXCLUDE ex);
E2_Action *e2_action_register (const gchar *name, E2_ACTION_TYPE type, void *func,
	gpointer data, gboolean has_arg);
E2_Action *_e2_action_register (const gchar *name, E2_ACTION_TYPE type, void *func,
	gpointer data, gboolean has_arg, E2_ACTION_EXCLUDE exclude,
	gpointer data2);
gboolean e2_action_unregister (const gchar *name);
void _e2_action_unregister (E2_Action *action);
E2_Action *e2_action_get (gchar *name);
gboolean e2_action_run_simple (gchar *name, gpointer arg);
gboolean e2_action_run_simple_from (gchar *name, gpointer arg,
	GtkWidget *from);
gboolean e2_action_run (GtkWidget *widget, E2_ActionRuntime *rt);
gboolean e2_action_run_from (E2_ActionRuntime *rt, GtkWidget *from);
E2_ActionRuntime *e2_action_pack_runtime (E2_Action *action, gpointer data,
	gpointer data_free);
void e2_action_free_runtime (E2_ActionRuntime *rt);

void e2_actions_clean ();
void e2_actions_init ();

#endif /* ifndef E2_ACTION_H */

