/* $Id: config_files.c,v 1.26 2004/05/18 12:04:50 cairo Exp $
 * 
 * emelFM
 * Copyright (C) 1999 Michael Clark.
 *
 * emelFM2 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <utime.h>
#include "emelfm2.h"
#include "build.h"
#include "debug.h"
#include "e2_option.h"
#include "e2_cl_option.h"
#include "e2_window.h"

#define parse_bool(a, b) \
		if (strstr(line, a)) \
		{ \
			value = (strchr(line, '=')+1); \
			b = g_str_equal(value, "TRUE"); \
			continue; \
		}

#define parse_str(a, b) \
		if (strstr(line, a)) \
		{ \
			value = (strchr(line, '=')+1); \
			strncpy(b, value, sizeof(b)); \
			continue; \
		}

#define parse_int(a, b) \
		if (strstr(line, a)) \
		{ \
			value = (strchr(line, '=')+1); \
			b = atoi(value); \
			continue; \
		}

gint check_config_files (gpointer data)
{
	struct stat stat_buf;

	if (stat(e2_cl_options.config_dir, &stat_buf) == 0)
	{
		if (stat_buf.st_mtime != cfg.config_mtime)
		{
			//config has changed
			sleep (1);
			printd (INFO, "reloading config due to external change");

			e2_cd_cancel_cb (NULL);
			e2_option_clear ();
			e2_keybinding_clean ();

			e2_option_read_from_file (DEFAULT_CONFIG_FILE);
			read_config_file();
			read_filetypes_file();
			read_plugins_file();
			cfg.config_mtime = stat_buf.st_mtime;

			e2_window_recreate (&app.window);
		}
	}

	return TRUE;
}

//this function should be called once we are done writing all config files
//to let any other instances know that they need to re-read the files
void touch_config_dir()
{
	struct stat stat_buf;

	if (utime(e2_cl_options.config_dir, NULL) == 0)
	{
		if (stat(e2_cl_options.config_dir, &stat_buf) == 0)
			cfg.config_mtime = stat_buf.st_mtime;
		else
		{
			status_errno();
			printd (WARN, "couldn't stat the config dir");
		}
	}
	else
	{
		status_errno();
		printd (WARN, "couldn't touch the config dir");
	}
}

void
write_filetypes_file()
{
	FILE *f;
	gchar filetypes_file[PATH_MAX+10];
	GList *tmp;

	g_snprintf(filetypes_file, sizeof(filetypes_file),
						 "%s/filetypes", e2_cl_options.config_dir);

	if ( (f = fopen(filetypes_file, "w")) == NULL )
	{
		fprintf(stderr, "Unable to open filetypes file for write\n");
		return;
	}

	for (tmp = cfg.filetypes; tmp != NULL; tmp = tmp->next)
	{
		FileType *ft = tmp->data;

		fprintf(f, "%s;%s;%s\n", ft->description, ft->extensions, ft->actions);
	}
	fclose(f);
}

gboolean
read_filetypes_file()
{
	FILE *f;
	gchar filetypes_file[PATH_MAX+10];
	gchar line[MAX_LEN];
	gchar *extensions;
	gchar *actions;
	gchar *description;
	gchar *s;

	g_snprintf(filetypes_file, sizeof(filetypes_file), "%s/filetypes", e2_cl_options.config_dir);
	if ( (f = fopen(filetypes_file, "r")) == NULL )
		return FALSE;

	e2_list_free_with_data(&cfg.filetypes);
	while ( fgets(line, MAX_LEN, f) )
	{
		if (line[0] == '\n') continue;

		if ((s = strstr(line, ";")) != NULL)
			*s = '\0';
		else
			continue;

		description = g_strdup(line);
		s++;
		extensions = s;

		if ((s = strstr(extensions, ";")) != NULL)
			*s = '\0';
		else
			continue;
		extensions = g_strdup(extensions);
		s++;
		actions = s;

		g_strchomp (actions);

		add_filetype(extensions, actions, description);

		g_free (description);
		g_free (extensions);
	}
	fclose(f);

	return TRUE;
}

void
write_plugins_file()
{
	FILE *f;
	gchar filename[PATH_MAX+NAME_MAX];
	GList *tmp;

	g_snprintf(filename, sizeof(filename), "%s/plugins", e2_cl_options.config_dir);
	if ((f = fopen(filename, "w")) == NULL)
	{
		fprintf(stderr, "Unable to open plugins file for writing\n");
		return;
	}

	for (tmp = cfg.plugins; tmp != NULL; tmp = tmp->next)
	{
		Plugin *p = tmp->data;

		fprintf(f, "%s:%s\n", p->filename, (p->show_in_menu ? "TRUE" : "FALSE"));
	}
	fclose(f);
}

gint
read_plugins_file()
{
	FILE *f;
	gchar filename[PATH_MAX+NAME_MAX];
	gchar plugin_path[PATH_MAX+NAME_MAX], *s;
	Plugin *p;

	g_snprintf(filename, sizeof(filename), "%s/plugins", e2_cl_options.config_dir);
	if ( (f = fopen(filename, "r")) == NULL )
		return FALSE;

	unload_all_plugins();
	while ( fgets(plugin_path, sizeof(plugin_path), f) )
	{
		g_strchomp (plugin_path);
		if ((s = strrchr(plugin_path, ':')) != NULL)
		{
			*s++ = '\0';
			if ((p = load_plugin(plugin_path)) != NULL)
			{
				p->show_in_menu = g_str_equal(s, "TRUE");
				cfg.plugins = g_list_append(cfg.plugins, p);
			}
		}
		else
		{
			if ((p = load_plugin(plugin_path)) != NULL)
				cfg.plugins = g_list_append(cfg.plugins, p);
		}
	}
	fclose(f);

	return TRUE;
}

void
write_config_file()
{
	FILE *f;
	gchar config_file[PATH_MAX+10];
	int i;

	g_snprintf(config_file, sizeof(config_file), "%s/settings", e2_cl_options.config_dir);
	if ((f = fopen(config_file, "w")) == NULL)
	{
		fprintf(stderr, "Unable to open config file for writing\n");
		return;
	}

	fprintf(f, "VERSION=%s\n", VERSION);
	fprintf(f, "SHOW_HIDDEN_LEFT=%s\n", (app.left_view.show_hidden ? "TRUE" : "FALSE"));
	fprintf(f, "SHOW_HIDDEN_RIGHT=%s\n", (app.right_view.show_hidden ? "TRUE" : "FALSE"));
	fprintf(f, "WINDOW_HEIGHT=%d\n", app.main_window->allocation.height);
	fprintf(f, "WINDOW_WIDTH=%d\n", app.main_window->allocation.width);

	for (i = 0; i < MAX_COLUMNS; i++)
	{
		if (all_columns[i].is_visible)
		{
			gint r, l;
			r = GTK_CLIST(app.right_view.clist)->column[i].area.width;
			l = GTK_CLIST(app.left_view.clist)->column[i].area.width;
			fprintf(f, "COLUMN=%d,%d\n", i, (r > l ? r : l));
		}
	}

	fclose(f);
}

gboolean
read_config_file()
{
	FILE *f;
	gchar config_file[PATH_MAX+10];
	gchar line[MAX_LEN];
	gchar *value;
	int clear_default_columns = TRUE;

	g_snprintf(config_file, sizeof(config_file), "%s/settings", e2_cl_options.config_dir);
	if ((f = fopen(config_file, "r")) == NULL)
		return FALSE;

	while (fgets(line, MAX_LEN, f))
	{
		g_strchomp (line);

		parse_str("VERSION", cfg.version);
		parse_bool("SHOW_HIDDEN_LEFT", app.left_view.show_hidden);
		parse_bool("SHOW_HIDDEN_RIGHT", app.right_view.show_hidden);
		parse_int("WINDOW_HEIGHT", cfg.window_height);
		parse_int("WINDOW_WIDTH", cfg.window_width);

		if (strstr(line, "COLUMN"))
		{
			int i;
			gchar *s;
			if (clear_default_columns)
			{
				for (i = 0; i < MAX_COLUMNS; i++)
					all_columns[i].is_visible = FALSE;
				clear_default_columns = FALSE;
			}
			value = (strchr(line, '=')+1);
			if ((s = strchr(value, ',')) != NULL)
			{
				/* Now we store the size also */
				*s++ = '\0';
				i = atoi(value);
				all_columns[i].is_visible = TRUE;
				all_columns[i].size = atoi(s);
			}
			else
			{
				/* No size here.. just the col */
				i = atoi(value);
				all_columns[i].is_visible = TRUE;
			}
			continue;
		}
	}
	fclose(f);

	return TRUE;
}

