/* $Id: e2_option_tree.h,v 1.7 2004/08/07 21:57:53 cairo Exp $
 * 
 * Copyright (C) 2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef E2_OPTION_TREE_H
#  define E2_OPTION_TREE_H

#include "stdio.h"
#include "emelfm2.h"
#include "e2_option.h"

typedef enum
{
	E2_OPTION_TREE_TYPE_BOOL,
	E2_OPTION_TREE_TYPE_STR,
	E2_OPTION_TREE_TYPE_ICON,
	E2_OPTION_TREE_TYPE_SEL,
	E2_OPTION_TREE_TYPE_KEY
} E2_OptionTreeType;

typedef enum
{
	E2_OPTION_TREE_SET_NOT_EDITABLE = 1<<0,
} E2_OptionTreeSetFlags;

typedef struct _E2_OptionTreeSet
{
	gchar *name;
	E2_OptionTreeType type;
	gint idef;
	gchar *sdef;
	void *extra;
	void *extra_data;
	E2_OptionTreeSetFlags flags;
	gpointer visible_check_func;
	gpointer visible_check_func_data;
} E2_OptionTreeSet;

void e2_option_tree_add_widget (GtkWidget *parent, GtkWidget *box,
	E2_OptionSet *set);
E2_OptionSet *e2_option_tree_register (gchar *name, gchar *group, gchar *desc,
	gchar *depends, gpointer selection_check_func, gpointer draggable_check_func,
	E2_OptionTypeTreeFlags flags);
gint e2_option_tree_register_unknown (gchar *name, gint i, gchar *buf[]);
E2_OptionTreeSet *e2_option_tree_add_column (E2_OptionSet *set, gchar *name,
	E2_OptionTreeType type, gint idef, gchar *sdef, void *extra,
	void *extra_data, E2_OptionTreeSetFlags flags, gpointer visible_check_func,
	gpointer visible_check_func_data);
void e2_option_tree_sync_columns (E2_OptionSet *set);
void e2_option_tree_revert (gchar *name);
void _e2_option_tree_revert (E2_OptionSet *set, gboolean empty_check);
void _e2_option_tree_append_default (E2_OptionSet *set, gchar *first, ...);

E2_OptionSet *_e2_option_tree_check_helper (gchar *option);
void e2_option_tree_add (gchar *option, GtkTreeIter *iter, gint n_options,
	void *options[]);
void _e2_option_tree_add (E2_OptionSet *set, GtkTreeIter *iter,
	GtkTreeIter *parent, gboolean sibling, gboolean before, gint n_options,
	void *options[]);
void e2_option_tree_add_default (gchar *option, GtkTreeIter *iter,
	GtkTreeIter *parent, gboolean sibling);
void _e2_option_tree_add_default (E2_OptionSet *set, GtkTreeIter *iter,
	GtkTreeIter *parent, gboolean sibling);

void e2_option_tree_del (gchar *option, GtkTreeIter *iter);
void _e2_option_tree_del (E2_OptionSet *set, GtkTreeIter *iter);

void _e2_option_tree_move (E2_OptionSet *set, GtkTreeIter *iter,
	GtkTreePath *path_dest, gboolean sibling, gboolean before);

void e2_option_tree_set (gchar *option, GtkTreeIter *iter, gint col,
	void *data);
void _e2_option_tree_set (E2_OptionSet *set, GtkTreeIter *iter, gint col,
	void *data);

void *_e2_option_tree_get_void (E2_OptionTreeSet *set, gchar *option);

void _e2_option_tree_write_to_file (FILE *f, E2_OptionSet *set,
	GtkTreeIter *iter, gint level);

#endif /* ifndef E2_OPTION_TREE_H */

