/* $Id: e2_dialog.c,v 1.19 2004/08/01 12:38:50 cairo Exp $
 * 
 * Copyright (C) 2003-2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/**
 * @file src/dialogs/e2_dialog.c 
 * @brief dialog utility functions
 *
 * This file contains dialog utility functions.
 **/

#include <stdio.h>
#include "emelfm2.h"
#include "e2_dialog.h"
#include "e2_option.h"
#include "e2_button.h"
#include "build.h"

  /*********************/
 /***** callbacks *****/
/*********************/

static gboolean context_menu_cb (GtkWidget *widget, GdkEventButton *event,
	GtkWidget *menu)
{
	if (event->button == 3)
	{
		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
			event->button, event->time);
		return TRUE;
	}
	return FALSE;
}

static void show_cb (GtkWidget *dialog, GtkScrolledWindow *scrolled)
{
	gtk_scrolled_window_set_policy (scrolled, GTK_POLICY_AUTOMATIC,
		GTK_POLICY_AUTOMATIC);
}

static void resize_destroy_cb (GtkWidget *dialog, gpointer data)
{
	gint idle_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (dialog),
			"e2-dialog-resize-idle-id"));
	if (idle_id != 0)
		g_source_remove (idle_id);
}

static gboolean resize_idle2_cb (GtkWidget *dialog)
{
	GtkScrolledWindow *scrolled = g_object_get_data (G_OBJECT (dialog),
		"e2-dialog-resize-scrolled");
	gtk_scrolled_window_set_policy (scrolled, GTK_POLICY_AUTOMATIC,
		GTK_POLICY_AUTOMATIC);
	//set idle id to 0, we're done with this resize cycle
	g_object_set_data (G_OBJECT (dialog), "e2-dialog-resize-idle-id",
		GINT_TO_POINTER (0));
	return FALSE;

}

static gboolean resize_idle_cb (GtkWidget *dialog)
{
	if ((dialog->allocation.width < dialog->requisition.width) ||
		(dialog->allocation.height < dialog->requisition.height))
	{
		gtk_window_resize (GTK_WINDOW (dialog),
			dialog->requisition.width, dialog->requisition.height);
	//set idle id to the 2nd half of the resize cycle
		guint idle_id = g_timeout_add (750, (GSourceFunc) resize_idle2_cb,
			dialog);
		g_object_set_data (G_OBJECT (dialog), "e2-dialog-resize-idle-id",
			GINT_TO_POINTER (idle_id));
	} else
		resize_idle2_cb (dialog);
	return FALSE;
}

  /******************/
 /***** public *****/
/******************/

//old
void prepare_dialog ()
{
	e2_widget_set_cursor  (GDK_WATCH);
	gtk_widget_set_sensitive (app.main_window, FALSE);
	disable_refresh ();
}

//old
void postpare_dialog (gboolean refresh, FileView *view)
{
	gtk_widget_set_sensitive (app.main_window, TRUE);
	if (refresh)
		refresh_list (view);
	else
		gtk_widget_grab_focus (curr_view->clist);

	e2_widget_set_cursor  (GDK_LEFT_PTR);
	reenable_refresh ();
}

gboolean e2_dialog_key_neg_cb (GtkWidget *widget, GdkEventKey *event, gpointer data)
{
	printd (DEBUG, "e2_dialog_key_neg_cb (widget:_,event:_,data_)");
	if (event->keyval == GDK_Escape)
	{
		gint response = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (data),
				"e2-negative-response"));
		gtk_dialog_response (GTK_DIALOG (data), response);
		return TRUE;
	}
	return FALSE;
}

gboolean e2_dialog_key_pos_cb (GtkWidget *widget, GdkEventKey *event, gpointer data)
{
	printd (DEBUG, "e2_dialog_key_pos_cb (widget:_,event:_,data_)");
	if (event->keyval == GDK_Return)
	{
		gint response = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (data),
				"e2-positive-response"));
		gtk_dialog_response (GTK_DIALOG (data), response);
		return TRUE;
	}
	return FALSE;
}

void e2_dialog_setup (GtkWidget *dialog, GtkWidget *parent)
{
	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent));
	gtk_window_set_wmclass (GTK_WINDOW (dialog),
		gtk_window_get_title (GTK_WINDOW (dialog)), PROGNAME);
	gtk_window_set_role (GTK_WINDOW (dialog), "dialog");
	gtk_window_set_position (GTK_WINDOW (dialog),
		e2_option_int_get ("dialog-position"));
}

gint e2_dialog_show (GtkWidget *dialog, GtkWidget *parent, gint buttons,
	E2_DialogFlags flags)
{
	e2_dialog_add_button (dialog, buttons);
	e2_dialog_setup (dialog, parent);
	if ((flags & E2_DIALOG_RUN) && (parent != NULL))
		gtk_widget_set_sensitive (parent, FALSE);

	if (flags & E2_DIALOG_DONT_SHOW_ALL)
		gtk_widget_show (dialog);
	else
		gtk_widget_show_all (dialog);
	
	gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
	gtk_widget_set_size_request (dialog, 0, 0);
	g_signal_connect (dialog, "key_press_event",
		G_CALLBACK (e2_dialog_key_neg_cb), dialog);
	g_signal_connect_after (dialog, "key_press_event",
		G_CALLBACK (e2_dialog_key_pos_cb), dialog);
	
	if (flags & E2_DIALOG_RUN)
	{
		gint ret = gtk_dialog_run (GTK_DIALOG (dialog));
		if (parent != NULL)
			gtk_widget_set_sensitive (parent, TRUE);
		if (flags & E2_DIALOG_FREE)
			gtk_widget_destroy (dialog);
		gtk_widget_grab_focus (curr_view->clist);
		return ret;
	}
	return 0;
}

//old
GtkWidget *get_dialog (GtkWidget *parent, gint type, gchar *label_text)
{
	GtkWidget *dialog;
	dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
		GTK_DIALOG_DESTROY_WITH_PARENT, type,
		GTK_BUTTONS_NONE, label_text);
	
	GtkLabel *label = GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label);
	gtk_label_set_use_markup (label, TRUE);
	GTK_WIDGET_UNSET_FLAGS (label, GTK_CAN_FOCUS);
	return dialog;
}

GtkWidget *e2_dialog_create (gchar *stock, gchar *label_text, gchar *title,
	void *func, void *data)
{
	GtkWidget *dialog = gtk_dialog_new ();

	gtk_widget_add_events (dialog, GDK_ALL_EVENTS_MASK);
	
	GtkWidget *hbox = gtk_hbox_new (FALSE, 5);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
	gboolean need_hbox = FALSE;
	
	g_object_set_data (G_OBJECT (dialog), "e2-dialog-hbox", hbox);
	
	GtkWidget *img = e2_widget_get_icon (stock, GTK_ICON_SIZE_DIALOG);
	if (img != NULL)
	{
		gtk_misc_set_alignment (GTK_MISC (img), 0.5, 0.0);
		gtk_box_pack_start (GTK_BOX (hbox), img, FALSE, FALSE, 0);
		need_hbox = TRUE;
	}

	if (label_text != NULL)
	{
		e2_widget_add_label (hbox, label_text, 0.5, TRUE, 0);
		need_hbox = TRUE;
	}

	if (need_hbox == TRUE)
	{
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, FALSE, FALSE, 0);
		gtk_widget_show_all (hbox);
	} else
		gtk_widget_destroy (hbox);

	if (title != NULL)
	    gtk_window_set_title (GTK_WINDOW (dialog), title);
	
	if (func != NULL)
		g_signal_connect (GTK_OBJECT (dialog),
			"response", G_CALLBACK (func), data);
	
	e2_dialog_set_positive_response (dialog, GTK_RESPONSE_OK);
	e2_dialog_set_negative_response (dialog, GTK_RESPONSE_CANCEL);

	return dialog;
}

GtkWidget *e2_dialog_add_sw (GtkWidget *dialog)
{
	GtkWidget *scrolled = e2_widget_add_sw (GTK_DIALOG (dialog)->vbox,
		GTK_POLICY_NEVER, GTK_POLICY_NEVER, TRUE, 0);
	GtkWidget *vbox = e2_widget_get_box (TRUE, FALSE, 0);
	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled), vbox);
	gtk_viewport_set_shadow_type (GTK_VIEWPORT (GTK_BIN (scrolled)->child),
		GTK_SHADOW_NONE);
	g_signal_connect (G_OBJECT (GTK_DIALOG (dialog)), "show",
		G_CALLBACK (show_cb), scrolled);
	GtkWidget *hbox = g_object_get_data (G_OBJECT (dialog), "e2-dialog-hbox");
	gtk_widget_reparent (hbox, vbox);
	gtk_box_set_child_packing (GTK_BOX (vbox), hbox, FALSE, FALSE, 0,
		GTK_PACK_START);
	gtk_widget_show (vbox);
	return vbox;
}

//adds predefined buttons to a dialog and returns last added button
GtkWidget *e2_dialog_add_button (GtkWidget *dialog, E2_ButtonType buttons)
{
	GtkWidget *button = NULL;
	if (buttons & E2_BUTTON_NO)
		button = _e2_dialog_add_button (dialog, GTK_STOCK_NO, "_no", GTK_RESPONSE_NO);
	if (buttons & E2_BUTTON_CANCEL)
		button = _e2_dialog_add_button (dialog, GTK_STOCK_CANCEL, "_cancel", GTK_RESPONSE_CANCEL);
	if (buttons & E2_BUTTON_REMOVE)
		button = _e2_dialog_add_button (dialog, GTK_STOCK_DELETE, "_remove", 110);
	if (buttons & E2_BUTTON_CREATE)
		button = _e2_dialog_add_button (dialog, GTK_STOCK_YES, "crea_te", 24);
	if (buttons & E2_BUTTON_APPLY_TO_ALL)
		button = _e2_dialog_add_button (dialog, GTK_STOCK_APPLY, "appl_y to all", 112);
	if (buttons & E2_BUTTON_APPLY)
		button = _e2_dialog_add_button (dialog, GTK_STOCK_APPLY, "_apply", GTK_RESPONSE_APPLY);
	if (buttons & E2_BUTTON_YES_TO_ALL)
		button = _e2_dialog_add_button (dialog, GTK_STOCK_YES, "ye_s to all", 111);
	if (buttons & E2_BUTTON_YES)
	{
		button = _e2_dialog_add_button (dialog, GTK_STOCK_YES, "_yes", GTK_RESPONSE_YES);
		e2_dialog_set_positive_response (dialog, GTK_RESPONSE_YES) ;
	}
	if (buttons & E2_BUTTON_OK)
	{
		button = _e2_dialog_add_button (dialog, GTK_STOCK_OK, "_ok", GTK_RESPONSE_OK);
		e2_dialog_set_positive_response (dialog, GTK_RESPONSE_OK);
	}

	return button;
}

GtkWidget *_e2_dialog_add_button (GtkWidget *dialog, gchar *stock,
	gchar *label, gint response)
{
	GtkWidget *button = e2_button_get (label, stock, NULL, NULL, NULL);
	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, response);
	gtk_widget_show_all (button);
	return button;
}

GtkWidget *_e2_dialog_add_toggle_button (GtkWidget *dialog, gboolean value,
	gchar *label, gint response)
{
	GtkWidget *button = e2_button_get_toggle (label, value, NULL, NULL, FALSE);
	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, response);
	gtk_widget_show_all (button);
	return button;
}

GtkWidget *_e2_dialog_add_check_button (GtkWidget *dialog, gboolean value,
	gchar *label, gint response)
{
	GtkWidget *button = e2_button_get_toggle (label, value, NULL, NULL, TRUE);
	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, response);
	gtk_widget_show_all (button);
	return button;
}

void e2_dialog_attach_menu (GtkWidget *dialog, GtkWidget *menu)
{
	g_return_if_fail ((dialog != NULL) && (menu != NULL));
	g_signal_connect (G_OBJECT (dialog), "button-press-event",
		G_CALLBACK (context_menu_cb), menu);
	g_object_set_data_full (G_OBJECT (dialog), "e2-context-menu",  menu,
		(GDestroyNotify) gtk_widget_destroy);
}

void e2_dialog_set_positive_response (GtkWidget *dialog, gint response)
{
	g_object_set_data (G_OBJECT (dialog), "e2-positive-response",
		GINT_TO_POINTER (response));
	gtk_dialog_set_default_response (GTK_DIALOG (dialog), response);
}

void e2_dialog_set_negative_response (GtkWidget *dialog, gint response)
{
	g_object_set_data (G_OBJECT (dialog), "e2-negative-response",
		GINT_TO_POINTER (response));
}

void e2_dialog_set_responses (GtkWidget *dialog, gint pos, gint neg)
{
	e2_dialog_set_positive_response (dialog, pos);
	e2_dialog_set_negative_response (dialog, neg);
}

void e2_dialog_resize (GtkWidget *dialog, gfloat factor)
{
	gint width, height;
	gtk_window_get_size (GTK_WINDOW (dialog), &width, &height);
	width *= factor;
	height *= factor;
	gtk_window_resize (GTK_WINDOW (dialog), width, height);
}

void e2_dialog_resize_with_sw (GtkWidget *dialog, GtkWidget *scrolled)
{
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
		GTK_POLICY_NEVER, GTK_POLICY_NEVER);
	g_object_set_data (G_OBJECT (dialog), "e2-dialog-resize-scrolled",
		scrolled);
	gint idle_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (dialog),
			"e2-dialog-resize-idle-id"));
	if (idle_id != 0)
		g_source_remove (idle_id);
	idle_id = g_idle_add ((GSourceFunc) resize_idle_cb, dialog);
	g_object_set_data (G_OBJECT (dialog), "e2-dialog-resize-idle-id",
		GINT_TO_POINTER (idle_id));
	g_signal_connect (G_OBJECT (dialog), "destroy",
		G_CALLBACK (resize_destroy_cb), NULL);
}

void e2_dialog_init ()
{
	e2_action_register ("dialog.about", E2_ACTION_TYPE_ITEM,
		e2_about_dialog_show, NULL, FALSE);
	gchar *opt_dialog_pos_position[] = {"none", "center", "mouse",
		"always center", "center on parent", NULL};
	e2_option_sel_register ("dialog-position", "interface:misc", "dialog position",
		"default position where dialogs should popup", NULL, 1,
		opt_dialog_pos_position);

	e2_mkdir_dialog_init ();
	e2_search_dialog_init ();
	e2_view_dialog_init ();
}
