/* $Id: e2_mkdir_dialog.c,v 1.38 2004/07/27 22:48:39 cairo Exp $
 * 
 * Copyright (C) 2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "emelfm2.h"
#include "e2_mkdir_dialog.h"

static GList *history = NULL;
static E2_OptionSet *opt_connected;
static E2_OptionSet *opt_show_info;
static E2_OptionSet *opt_show_last;
static E2_OptionSet *opt_suggest_dir;
static E2_OptionSet *opt_follow_pane;

static gboolean changed_cb_idle (E2_MkdirDialogRuntime *rt);
static gboolean changed_cb (GtkWidget *entry, E2_MkdirDialogRuntime *rt);
static gboolean change_dir_cb (gchar *path, E2_MkdirDialogRuntime *rt);
static gboolean change_focus_cb (E2_PaneRuntime *pane_rt, E2_MkdirDialogRuntime *rt);

  /*****************/
 /***** utils *****/
/*****************/

static void dialog_destroy (E2_MkdirDialogRuntime *rt)
{
	gtk_widget_destroy (rt->dialog);
	e2_list_free_with_data (&rt->history);
	e2_hook_unregister (&app.pane1.hook_change_dir, change_dir_cb, rt, TRUE);
	e2_hook_unregister (&app.pane2.hook_change_dir, change_dir_cb, rt, TRUE);
	e2_hook_unregister (&app.hook_pane_focus_changed, change_focus_cb, rt, TRUE);
	g_free (rt->path);
	if (rt->idle_id != 0)
		g_source_remove (rt->idle_id);
	g_free (rt);
}

static void update_dialog_size (E2_MkdirDialogRuntime *rt)
{
	if ((!GTK_IS_WIDGET (rt->dialog)) || (!GTK_WIDGET_VISIBLE (rt->dialog)))
		return;
	e2_dialog_resize_with_sw (rt->dialog, rt->scrolled);
}

//tries to suggests a directory name by adding or incrementing a number
static gchar *find_dir (const gchar *dir, const gchar *parent_dir)
{
	//if we have to suggest something, but there is nothing in the
	//history yet, we use a default string
	if (dir == NULL)
		dir = _("new directory");
	//the absolute directory name consists of 4 parts
	gchar *str = _UTF2STR (dir);
	//parent directory in case the entered string is not absolute
	gchar *part1 = g_path_is_absolute (str) ?
		g_strdup ("") : _UTF2STR (parent_dir);
	//in case the user entered something like "dir/blah"
	gchar *part2 = g_path_get_dirname (str);
	gchar *part3 = g_path_get_basename (str);
	g_free (str);
	//last part to hold the rest of part 3 after we found a number
	gchar *part4 = "";
	//postion of the start of an existing number in part3
	gint digit_start = -1;
	//digit to increment or add to the directory name
	guint i = 0;
	//complete dir
	gchar *path = g_build_filename (part1, part2, part3, part4, NULL);

	gboolean have_to_search = FALSE;
	//to simply stuff, we simply do this check 2 times
	//this is the first time, if the directory the user
	//tries to create does already exist, it searches for a number
	//in part 3 and sets up part4 (to the string after the number)
	if (g_file_test (path, G_FILE_TEST_EXISTS))
	{
		have_to_search = TRUE;
		gint j = strlen (part3);
		//find first digit starting at the end of the string
		gboolean last_was_digit = FALSE;
		while (j >= 0)
		{
			if (g_ascii_isdigit (part3[j]))
			{
				if (!last_was_digit)
					last_was_digit = TRUE;
				if (j == 0)
					digit_start = 0;
			} else if (last_was_digit)
			{
				digit_start = j + 1;
				break;
			}
			j--;
		}
		if (digit_start != -1)
		{
			while (*(part3 + digit_start) == '0')
				digit_start++;
			if (!g_ascii_isdigit (*(part3 + digit_start)))
				digit_start--;
			i = g_ascii_strtoull (part3 + digit_start, &part4, 10);
			part3[digit_start] = '\0';
		}
	}

	//the real test loop that increments i
	while (g_file_test (path, G_FILE_TEST_EXISTS))
	{
		i++;
		g_free (path);
		if (digit_start != -1)
			path = g_strdup_printf ("%s/%s/%s%d%s", part1, part2, part3, i, part4);
		//if there is no digit/number in the directory string, we simply add it
		//to the end, ignoring part 4
		else
			path = g_strdup_printf ("%s/%s/%s %d", part1, part2, part3, i);
	}
	g_free (path);
	path = NULL;
	
	//now build the string for the entry
	if (have_to_search)
	{
		//build the retval from the the string the user really entered
		//(dir argument)
		if (digit_start != -1)
		{
			path = g_strdup_printf ("%s%d%s", part3, i, part4);
			part3[digit_start] = '0';
		}
		else
			path = g_strdup_printf ("%s %d", part3, i);
	
		//add part2 if necessary
		if (!((part2[0] == '.') && (part2[1] == '\0')))
		{
			gchar *free = path;
			path = g_build_filename (part2, path, NULL);
			g_free (free);
		}
	}
	
	//cleanup
	g_free (part1);
	g_free (part2);
	g_free (part3);
	gchar *utf = have_to_search ? _STR2UTF (path) : g_strdup (dir);
	g_free (path);
	return utf;
}

static void update_combo (E2_MkdirDialogRuntime *rt)
{
	GtkWidget *entry = GTK_BIN (rt->combo)->child;
	if (rt->opt_suggest_dir || rt->opt_show_last)
	{
		GtkTreeIter iter;
		gchar *text = e2_tree_get_last (gtk_combo_box_get_model
			(GTK_COMBO_BOX (rt->combo)), 0, &iter);
		gchar *new_text = NULL;
		if (rt->opt_suggest_dir)
			new_text = find_dir (text, rt->path);
		else
			new_text = g_strdup (text);
		if ((text != NULL) && (g_str_equal (text, new_text)))
			e2_combobox_set_active (rt->combo, e2_tree_count (gtk_combo_box_get_model
				(GTK_COMBO_BOX (rt->combo))) - 1);
		g_free (text);
		if (new_text == NULL)
			gtk_entry_set_text (GTK_ENTRY (entry), "");
		else
		{
			gtk_entry_set_text (GTK_ENTRY (entry), new_text);
			g_free (new_text);
		}
		gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
	} else
		gtk_entry_set_text (GTK_ENTRY (entry), "");
}

static gchar *real_parent (gchar *parent, gboolean *exists, gint *i)
{
	//find last existing parent directory
	while ((!g_file_test (parent, G_FILE_TEST_EXISTS)) ||
		(!g_file_test (parent, G_FILE_TEST_IS_DIR)))
	{
		(*i)++;
		*exists = FALSE;
		if ((parent[0] == '/') && (parent[1] == '\0'))
			break;
		gchar *free = parent;
		parent = g_path_get_dirname (parent);
		g_free (free);
	}
	return parent;
}
static void _update_label1 (gchar *path, E2_MkdirDialogRuntime *rt)
{
	gchar *translated = e2_translate_to_filename (rt->path, path);
	gchar *locale_encoded = _STR2UTF (translated);
	g_free (translated);
	gchar *label = g_strconcat ("<small>", locale_encoded, "</small>", NULL);
	g_free (locale_encoded);
	gtk_label_set_markup (GTK_LABEL (rt->info_label), label);
	g_free (label);
}

static void _update_label2 (gchar *path, gchar *reason, E2_MkdirDialogRuntime *rt)
{
	gchar *color;
	gchar *constant;
	if (rt->creation_possible)
	{
		constant = _("yes");
//		gtk_widget_set_sensitive (rt->button_create, TRUE);
		gtk_dialog_set_response_sensitive (GTK_DIALOG (rt->dialog), 24, TRUE);
		color = e2_option_str_get ("color-positive");
	} else
	{
		constant = _("no");
//		gtk_widget_set_sensitive (rt->button_create, FALSE);
		gtk_dialog_set_response_sensitive (GTK_DIALOG (rt->dialog), 24, FALSE);
		color = e2_option_str_get ("color-negative");
	}
	gchar *label = g_strconcat ("<span weight=\"bold\" size=\"small\" foreground=\"",
			color, "\">", constant, " </span>", reason == NULL ? NULL : "<small>(",
			reason, ")</small>", NULL);
	gtk_label_set_markup (GTK_LABEL (rt->info_label2), label);
	g_free (label);
}

//update the two info labels of the info frame
//this is done even if the info frame isn't shown to update the
//rt->creation_possible state
static void update_labels (E2_MkdirDialogRuntime *rt)
{
	rt->creation_possible = TRUE;

	const gchar *entry_text = gtk_entry_get_text (GTK_ENTRY
		(GTK_BIN (rt->combo)->child));

	//get locale encoded string
	gchar *path = _UTF2STR (entry_text);

	gboolean absolute = TRUE;
	gchar *parent;
	//if the entry is empty, we'll only check if the parent directory
	//is writable, thus, set path to the currently active directory
	if (entry_text[0] == '\0')
	{
		path = _UTF2STR (rt->path);
		parent = g_strdup (path);
	} else
	//otherwise we have to create an absolute path from the entry text
	//and the active directory
	{
		path = _UTF2STR (entry_text);
		absolute = g_path_is_absolute (path);
	
		//if the directory entered is not absolute, concatenate it
		if (!absolute)
		{
			gchar *path_str = _UTF2STR (rt->path);
			gchar *free = path;
			path = g_strconcat (path_str, G_DIR_SEPARATOR_S, path, NULL);
			g_free (path_str);
			g_free (free);
		}
		parent = g_path_get_dirname (path);
	}

	gboolean parent_exists = TRUE;
	gint num = g_str_has_suffix (entry_text, G_DIR_SEPARATOR_S) ? 0 : 1;
	parent = real_parent (parent, &parent_exists, &num);

	_update_label1 (parent, rt);
	
	gchar *reason = NULL;
	if ((access (parent, W_OK | X_OK)) != 0)
	{
		if (absolute)
			reason = g_strdup_printf (_("The directory '%s' is not writable: %s."),
				parent, g_strerror (errno));
		else if (parent_exists)
			reason = g_strdup_printf (_("Cannot write to parent directory: %s."),
				g_strerror (errno));
		else
			reason = g_strdup_printf (_("Only '%s' exists and is not writable: %s."),
				parent, g_strerror (errno));
		rt->creation_possible = FALSE;
		_update_label2 (path, reason, rt);
		g_free (path);
		g_free (parent);
		return;
	}
	
	if (entry_text[0] == '\0')
	{
		_update_label2 (path, reason, rt);
//		gtk_widget_set_sensitive (rt->button_create, FALSE);
		gtk_dialog_set_response_sensitive (GTK_DIALOG (rt->dialog), 24, FALSE);
		g_free (path);
		g_free (parent);
		return;
	}
	
	if (g_file_test (path, G_FILE_TEST_EXISTS))
	{
		if  (g_file_test (path, G_FILE_TEST_IS_DIR))
			reason = _("The directory already exists.");
		else
			reason = _("Something is in the way.");
		
		rt->creation_possible = FALSE;
	}

	if (num > 1)
		reason = g_strdup_printf (_("%d directories will be created"), num);
	_update_label2 (path, reason, rt);
	g_free (parent);
	g_free (path);
}

static void update_follow_dir (E2_MkdirDialogRuntime *rt, gboolean follow)
{
	if (follow)
	{
		g_free (rt->path);
		rt->path = g_strdup (app.pane_current->path);
		update_labels (rt);
		e2_hook_register (&app.pane1.hook_change_dir, change_dir_cb, rt);
		e2_hook_register (&app.pane2.hook_change_dir, change_dir_cb, rt);
		e2_hook_register (&app.hook_pane_focus_changed, change_focus_cb, rt);
	} else
	{
		e2_hook_unregister (&app.pane1.hook_change_dir, change_dir_cb, rt, TRUE);
		e2_hook_unregister (&app.pane2.hook_change_dir, change_dir_cb, rt, TRUE);
		e2_hook_unregister (&app.hook_pane_focus_changed, change_focus_cb, rt, TRUE);
	}
}

static void update_info_box (E2_MkdirDialogRuntime *rt, gboolean resize, gboolean show)
{
	/*
	if (show)
	{
		if (!GTK_WIDGET_VISIBLE (rt->info_box))
		{
			gtk_widget_show (rt->info_box);
			update_labels (rt);
			gtk_widget_show (rt->info_box);
			if (resize)
				update_dialog_size (rt);
		}
	} else
	{
		if (GTK_WIDGET_VISIBLE (rt->info_box))
		{
			gtk_widget_hide (rt->info_box);
			if (resize)
				update_dialog_size (rt);
		}
	}
	*/
}

static void create_dir (GtkWidget *entry, E2_MkdirDialogRuntime *rt, gboolean close)
{
	if (!rt->creation_possible)
		return;
	const gchar *dir = gtk_entry_get_text (GTK_ENTRY (entry));
	//quick exit
	if (*dir == '\0')
		return;
	
	gchar *path = _UTF2STR (dir);
	if (!g_path_is_absolute (path))
	{
		gchar *free = path;
		gchar *cur = _UTF2STR (rt->path);
		path = g_strconcat (cur, path, NULL);
		g_free (free);
		g_free (cur);
	}
	
	gchar *parent = g_path_get_dirname (path);
	if (g_str_has_suffix (path, "/"))
	{
		gchar *free = parent;
		parent = g_path_get_dirname (parent);
		g_free (free);
	}

	GList *all = NULL;
	//find last existing parent directory
	while ((!g_file_test (parent, G_FILE_TEST_EXISTS)) ||
		(!g_file_test (parent, G_FILE_TEST_IS_DIR)))
	{
		if ((parent[0] == '/') && (parent[1] == '\0'))
			break;
		all = g_list_prepend (all, parent);
		parent = g_path_get_dirname (parent);
	}
	g_free (parent);
	
	GList *tmp;
	for (tmp = all; tmp != NULL; tmp = g_list_next (tmp))
	{
		if ((mkdir (tmp->data, 0777)) != 0)
		{
			gchar *message = g_strdup_printf (_("Could not create diretory '%s'. (%s.)\n"), path,
				g_strerror (errno));
			e2_output_print_error (message);
			g_free (message);
			update_labels (rt);
			refresh_list (curr_view);
			e2_list_free_with_data (&all);
			return;
		}
	}
	e2_list_free_with_data (&all);

	if ((mkdir (path, 0777)) == 0)
	{
		e2_list_update_history ((gchar *) dir, &history, NULL, 30, FALSE);
		update_combo (rt);
	} else
	{
		gchar *message = g_strdup_printf (_("Could not create diretory '%s'. (%s.)\n"), path,
			g_strerror (errno));
		e2_output_print_error (message);
		g_free (message);
	}

	if (!close)
		update_labels (rt);
	refresh_list (curr_view);
}

  /*********************/
 /***** callbacks *****/
/*********************/

static gboolean changed_cb_idle (E2_MkdirDialogRuntime *rt)
{
	update_dialog_size (rt);
	rt->idle_id = 0;
	return FALSE;
}

static gboolean changed_cb (GtkWidget *entry, E2_MkdirDialogRuntime *rt)
{
	update_labels (rt);
	if (rt->idle_id != 0)
		g_source_remove (rt->idle_id);
	rt->idle_id = g_idle_add ((GSourceFunc) changed_cb_idle, rt);
	return FALSE;
}

static void response_cb (GtkWidget *dialog, gint response,
	E2_MkdirDialogRuntime *rt)
{
	printd (DEBUG, "response_cb (dialog:_,response:%d,rt:_)", response);
	GtkWidget *entry = GTK_BIN (rt->combo)->child;
	switch (response)
	{
		case 24:
			e2_combobox_activate_cb (entry, GINT_TO_POINTER (FALSE));
			create_dir (entry, rt, FALSE);
			gtk_widget_grab_focus (entry);
			break;
		case GTK_RESPONSE_OK:
		{
			e2_combobox_activate_cb (entry, GINT_TO_POINTER (FALSE));
			create_dir (entry, rt, TRUE);
			dialog_destroy (rt);
			break;
		}
		default:
			dialog_destroy (rt);
			break;
	}
}

static void activate_cb (GtkWidget *entry, E2_MkdirDialogRuntime *rt)
{
	response_cb (rt->dialog, GTK_RESPONSE_OK, rt);
}

static gboolean change_dir_cb (gchar *path, E2_MkdirDialogRuntime *rt)
{
	g_free (rt->path);
	rt->path = g_strdup (path);
	update_labels (rt);
	update_dialog_size (rt);
	return TRUE;
}

static gboolean change_focus_cb (E2_PaneRuntime *pane_rt, E2_MkdirDialogRuntime *rt)
{
	g_free (rt->path);
	rt->path = g_strdup (pane_rt->path);
	update_labels (rt);
	update_dialog_size (rt);
	return TRUE;
}

static void toggled_cb1 (GtkWidget *menu_item, E2_MkdirDialogRuntime *rt)
{
	rt->opt_show_last = gtk_check_menu_item_get_active 
		(GTK_CHECK_MENU_ITEM (menu_item));
	update_combo (rt);
}

static void toggled_cb2 (GtkWidget *menu_item, E2_MkdirDialogRuntime *rt)
{
	rt->opt_suggest_dir = gtk_check_menu_item_get_active 
		(GTK_CHECK_MENU_ITEM (menu_item));
	update_combo (rt);
}

static void toggled_cb3 (GtkWidget *menu_item, E2_MkdirDialogRuntime *rt)
{
	update_follow_dir (rt, gtk_check_menu_item_get_active 
		(GTK_CHECK_MENU_ITEM (menu_item)));
}

static void toggled_expander_cb (GtkWidget *expander, GParamSpec *spec,
	E2_MkdirDialogRuntime *rt)
{
	update_dialog_size (rt);
}

static void show_cb (GtkWidget *dialog, E2_MkdirDialogRuntime *rt)
{
	update_combo (rt);
	gtk_expander_set_expanded (GTK_EXPANDER (rt->info_expander),
		_e2_option_bool_get (opt_show_info));
}

  /******************/
 /***** public *****/
/******************/

GtkWidget *e2_mkdir_dialog_create ()
{
	//init runtime object
	E2_MkdirDialogRuntime *rt = g_malloc (sizeof (E2_MkdirDialogRuntime));
	rt->history = e2_list_copy_with_data (history);
	rt->path = g_strdup (app.pane_current->path);
	rt->creation_possible = TRUE;
	rt->idle_id = 0;
	
	//create dialog
	rt->dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
		_("<b>What is the name for new directory?</b>"), _("create directory"),
		response_cb, rt);

	GtkWidget *vbox = e2_dialog_add_sw (rt->dialog);
	rt->scrolled = vbox->parent->parent;
	
	//add combo
	rt->combo = e2_combobox_add (vbox,
		FALSE, 5, activate_cb, rt, E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_FOCUS_ON_CHANGE);
	e2_combobox_append_history (rt->combo, history);

	//add info box with expander
	rt->info_box = e2_widget_get_box (TRUE, TRUE, TRUE);
	rt->info_expander = e2_option_bool_add_expander_widget (rt->dialog,
		vbox, _("info"), opt_show_info, rt->info_box);
	gtk_widget_show (rt->info_expander);
	g_signal_connect (G_OBJECT (rt->info_expander), "notify::expanded",
		G_CALLBACK (toggled_expander_cb), rt);

	GtkSizeGroup *group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
	GtkWidget *hbox = e2_widget_add_box (rt->info_box, FALSE, 0, FALSE, FALSE, 0);
	GtkWidget *label = _e2_widget_add_label (hbox,
		_("<small>parent directory:</small>"), 0.0, 0.0, FALSE, 3);
	gtk_size_group_add_widget (group, label);

	rt->info_label = e2_widget_add_label (hbox, "", 0.0, TRUE, 3);
	hbox = e2_widget_add_box (rt->info_box, FALSE, 0, FALSE, FALSE, 0);
	label = _e2_widget_add_label (hbox,
		_("<small>creation possible:</small>"), 0.0, 0.0, FALSE, 3);
	gtk_size_group_add_widget (group, label);
	rt->info_label2 = _e2_widget_add_label (hbox, "", 0.0, 0.0, TRUE, 3);
	
	
	//now add some buttons
	e2_dialog_add_button (rt->dialog, E2_BUTTON_CANCEL | E2_BUTTON_CREATE |
		E2_BUTTON_OK);

	rt->menu = e2_option_create_menu (rt->dialog, NULL,
		opt_connected, e2_menu_control_cb, rt->dialog,
		opt_follow_pane, toggled_cb3, rt,
//		opt_show_info, toggled_cb4, rt,
		opt_show_last, toggled_cb1, rt,
		opt_suggest_dir, toggled_cb2, rt,
		NULL);
	
	g_signal_connect (G_OBJECT (GTK_BIN (rt->combo)->child), "changed",
		G_CALLBACK (changed_cb), rt);
	g_signal_connect (G_OBJECT (rt->dialog), "show", G_CALLBACK (show_cb), rt);
	update_info_box (rt, FALSE, _e2_option_bool_get (opt_show_info));
	update_follow_dir (rt, _e2_option_bool_get (opt_follow_pane));

	e2_option_connect (rt->dialog, _e2_option_bool_get (opt_connected));
	e2_dialog_attach_menu (rt->dialog, rt->menu);

	rt->opt_show_last = _e2_option_bool_get (opt_show_last);
	rt->opt_suggest_dir = _e2_option_bool_get (opt_suggest_dir);
	
	return rt->dialog;
}

void e2_mkdir_dialog_show ()
{
	GtkWidget *dialog = e2_mkdir_dialog_create (app.main_window);
	e2_dialog_show (dialog, app.main_window, 0, E2_DIALOG_DONT_SHOW_ALL);
}

void e2_mkdir_dialog_init ()
{
	e2_action_register ("dialog.mkdir", E2_ACTION_TYPE_ITEM,
		e2_mkdir_dialog_show, NULL, FALSE);
	opt_connected = e2_option_bool_register ("dialog-mkdir-connected",
		"dialogs.mkdir:general", _("dialog connected"),
		_("When active, the mkdir dialog is connected to the option system "
			"and will replicate option changes to other connected mkdir dialogs. "
			"Otherwise option changes will be temporary to the current dialog."), NULL, TRUE);
	opt_show_info = e2_option_bool_register ("dialog-mkdir-show-info",
		"dialogs.mkdir:general", _("show info frame"),
		_("The info frame is shown on the mkdir dialog."), NULL, TRUE);
	opt_show_last = e2_option_bool_register ("dialog-mkdir-show-last",
		"dialogs.mkdir:general", _("show last directory name"),
		_("When opening the dialog or after creating a directory, the name of "
		"the last created directory is shown in the entry field."),
			"!dialog-mkdir-suggest-directory", FALSE);
	opt_suggest_dir = e2_option_bool_register 
		("dialog-mkdir-suggest-directory", "dialogs.mkdir:general",
		_("suggest new directory name"), _("When opening the dialog or after "
		"creating a directory, suggest a working name for the new directory "
		"based on the last created directory and by adding an increasing "
		"number to."), NULL, TRUE);
	opt_follow_pane = e2_option_bool_register 
		("dialog-mkdir-follow-pane", "dialogs.mkdir:general",
		_("follow pane dir"), _("The parent directory for new directories "
		"will be changed according to the directory of the active file pane."),
		NULL, TRUE);
	e2_cache_list_register ("mkdir-history", &history);
}
