/* $Id: e2_search_dialog.c,v 1.10 2004/07/28 00:22:50 cairo Exp $
 * 
 * Copyright (C) 2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "emelfm2.h"
#include "e2_search_dialog.h"
#include "e2_textiter.h"

static GList *history = NULL;
static E2_OptionSet *opt_connected;
static E2_OptionSet *opt_show_options;
static E2_OptionSet *opt_show_info;
static E2_OptionSet *opt_show_last;
static E2_OptionSet *opt_case_sensitive;
static E2_OptionSet *opt_search_backward;
static E2_OptionSet *opt_wrap;

  /*****************/
 /***** utils *****/
/*****************/

static void dialog_destroy (E2_SearchDialogRuntime *rt)
{
	g_object_steal_data (G_OBJECT (rt->textview), "e2-search-dialog");
	gtk_widget_destroy (rt->dialog);
	e2_list_free_with_data (&rt->history);
	g_free (rt);
}

static void update_dialog_size (E2_SearchDialogRuntime *rt)
{
	if ((!GTK_IS_WIDGET (rt->dialog)) || (!GTK_WIDGET_VISIBLE (rt->dialog)))
		return;
	e2_dialog_resize_with_sw (rt->dialog, rt->scrolled);
}

static void update_combo (E2_SearchDialogRuntime *rt)
{
	GtkWidget *entry = GTK_BIN (rt->combo)->child;
	if (opt_show_last)
	{
		GtkTreeIter iter;
		gchar *text = e2_tree_get_last (gtk_combo_box_get_model
			(GTK_COMBO_BOX (rt->combo)), 0, &iter);
		gchar *new_text = NULL;
		new_text = g_strdup (text);
		if ((text != NULL) && (g_str_equal (text, new_text)))
			e2_combobox_set_active (rt->combo, e2_tree_count (gtk_combo_box_get_model
				(GTK_COMBO_BOX (rt->combo))) - 1);
		g_free (text);
		if (new_text == NULL)
			gtk_entry_set_text (GTK_ENTRY (entry), "");
		else
		{
			gtk_entry_set_text (GTK_ENTRY (entry), new_text);
			g_free (new_text);
		}
		gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
	} else
		gtk_entry_set_text (GTK_ENTRY (entry), "");
}

static void attach_iters (GtkTextBuffer *buffer,
	GtkTextIter *start, GtkTextIter *end)
{
	GtkTextMark *mark = gtk_text_buffer_get_mark (buffer, "e2-search-start");
	if (mark == NULL)
		gtk_text_buffer_create_mark (buffer, "e2-search-start", start, FALSE);
	else
		gtk_text_buffer_move_mark (buffer, mark, start);
	mark = gtk_text_buffer_get_mark (buffer, "e2-search-end");
	if (mark == NULL)
		gtk_text_buffer_create_mark (buffer, "e2-search-end", end, FALSE);
	else
		gtk_text_buffer_move_mark (buffer, mark, end);
}

static void unattach_iters (GtkTextBuffer *buffer)
{
	GtkTextMark *mark = gtk_text_buffer_get_mark (buffer, "e2-search-start");
	if (mark != NULL)
		gtk_text_buffer_delete_mark (buffer, mark);
	mark = gtk_text_buffer_get_mark (buffer, "e2-search-end");
	if (mark != NULL)
		gtk_text_buffer_delete_mark (buffer, mark);
}

static gboolean search (GtkTextIter *search_iter, GtkTextIter *start,
	GtkTextIter *end, const gchar *find, E2_SearchDialogRuntime *rt)
{
	GtkSourceSearchFlags flags = GTK_SOURCE_SEARCH_TEXT_ONLY;
	if (!rt->opt_case_sensitive)
		flags |= GTK_SOURCE_SEARCH_CASE_INSENSITIVE;
	if (rt->opt_search_backward)
		return e2_iter_backward_search (search_iter, find,
			flags, start, end, NULL);
	else
		return e2_iter_forward_search (search_iter, find,
			flags, start, end, NULL);
}

  /*********************/
 /***** callbacks *****/
/*********************/

static void show_cb (GtkWidget *dialog, E2_SearchDialogRuntime *rt)
{
	gtk_expander_set_expanded (GTK_EXPANDER (rt->option_expander),
		_e2_option_bool_get (opt_show_options));
	gtk_expander_set_expanded (GTK_EXPANDER (rt->info_expander),
		_e2_option_bool_get (opt_show_info));
}

static void toggled_expander_cb (GtkWidget *expander, GParamSpec *spec,
	E2_SearchDialogRuntime *rt)
{
	update_dialog_size (rt);
}

static void response_cb (GtkWidget *dialog, gint response,
	E2_SearchDialogRuntime *rt)
{
	printd (DEBUG, "response_cb (dialog:_,response:%d,rt:_)", response);
	if ((response == 10) || (response == GTK_RESPONSE_OK))
	{
		// get the find string
		GtkWidget *entry = GTK_BIN (rt->combo)->child;
		const gchar *find = gtk_entry_get_text (GTK_ENTRY (entry));
		if (*find == '\0')
			return;
		e2_combobox_activate_cb (entry, GINT_TO_POINTER (FALSE));
		e2_list_update_history (find, &history, NULL, 30, FALSE);
		update_combo (rt);
		
		//check if the TextView's buffer has changed and get new TextIters
		GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (rt->textview));
/*		if (buffer != rt->textbuffer)
		{
			rt->textbuffer = buffer;
			unattach_iters (buffer);
			search_iter = NULL;
		}
*/		
		GtkTextMark *mark;
		if (rt->opt_search_backward)
			mark = gtk_text_buffer_get_mark (buffer, "e2-search-start");
		else
			mark = gtk_text_buffer_get_mark (buffer, "e2-search-end");

		GtkTextIter *search_iter = NULL;
		GtkTextIter iter;
		if (mark != NULL)
		{
			gtk_text_buffer_get_iter_at_mark (buffer, &iter, mark);
			search_iter = &iter;
		}	

		if (search_iter == NULL)
		{
			if (rt->opt_search_backward)
				gtk_text_buffer_get_end_iter (buffer, &iter);
			else
				gtk_text_buffer_get_start_iter (buffer, &iter);
			search_iter = &iter;
		}
		
		// search for the string
		GtkTextIter start, end;
		gboolean found = search (search_iter, &start, &end, find, rt);

		if (!found && rt->opt_wrap)
		{
			unattach_iters (buffer);
			GtkTextIter iter;
			if (rt->opt_search_backward)
				gtk_text_buffer_get_end_iter (buffer, &iter);
			else
				gtk_text_buffer_get_start_iter (buffer, &iter);
			found = search (&iter, &start, &end, find, rt);
		}
		if (found)
		{
			gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (rt->textview), &end,
					0.0, FALSE, 0.0, 0.0);
			gtk_text_buffer_select_range (buffer, &start, &end);
			attach_iters (buffer, &start, &end);
		}
		gchar *label = g_strconcat ("<span weight=\"bold\" size=\"small\" foreground=\"",
				e2_option_str_get (found ? "color-positive": "color-negative"),
				"\">", found ? "found" : "not found", "</span>", NULL);
		gtk_label_set_markup (GTK_LABEL (rt->info_label), label);
		g_free (label);
		gtk_widget_grab_focus (entry);
	} else
		dialog_destroy (rt);
}

static void activate_cb (GtkWidget *entry, E2_SearchDialogRuntime *rt)
{
	response_cb (rt->dialog, 10, rt);
}

static void toggled_cb1 (GtkWidget *menu_item, E2_SearchDialogRuntime *rt)
{
	rt->opt_show_last = gtk_check_menu_item_get_active 
		(GTK_CHECK_MENU_ITEM (menu_item));
	update_combo (rt);
}

static void toggled_cb2 (GtkWidget *menu_item, E2_SearchDialogRuntime *rt)
{
	rt->opt_case_sensitive = gtk_check_menu_item_get_active 
		(GTK_CHECK_MENU_ITEM (menu_item));
}

static void toggled_cb3 (GtkWidget *menu_item, E2_SearchDialogRuntime *rt)
{
	rt->opt_search_backward = gtk_check_menu_item_get_active 
		(GTK_CHECK_MENU_ITEM (menu_item));
}

static void toggled_cb4 (GtkWidget *menu_item, E2_SearchDialogRuntime *rt)
{
	rt->opt_wrap = gtk_check_menu_item_get_active 
		(GTK_CHECK_MENU_ITEM (menu_item));
}


  /******************/
 /***** public *****/
/******************/

GtkWidget *e2_search_dialog_create (GtkWidget *textview)
{
	//init runtime object
	E2_SearchDialogRuntime *rt = g_malloc (sizeof (E2_SearchDialogRuntime));
	rt->history = e2_list_copy_with_data (history);
	rt->textview = textview;
	rt->textbuffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
	rt->opt_show_last = _e2_option_bool_get (opt_show_last);
	rt->opt_case_sensitive = _e2_option_bool_get (opt_case_sensitive);
	rt->opt_search_backward = _e2_option_bool_get (opt_search_backward);
	rt->opt_wrap = _e2_option_bool_get (opt_wrap);

	g_object_set_data_full (G_OBJECT (textview), "e2-search-dialog", rt,
		(GDestroyNotify) dialog_destroy);
	
	//create dialog
	rt->dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
		_("<b>What do you want to search for?</b>"), _("search"), response_cb, rt);
	GtkWidget *vbox = e2_dialog_add_sw (rt->dialog);
	rt->scrolled = vbox->parent->parent;
	
	//add combo
	rt->combo = e2_combobox_add (vbox,
		FALSE, 5, activate_cb, rt, E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_FOCUS_ON_CHANGE);
	e2_combobox_append_history (rt->combo, history);
	update_combo (rt);

	rt->option_box = e2_widget_get_box (TRUE, TRUE, TRUE);
	rt->option_expander = e2_option_bool_add_expander_widget (rt->dialog,
		vbox, _("options"), opt_show_options, rt->option_box);
	gtk_widget_show (rt->option_expander);
	g_signal_connect (G_OBJECT (rt->option_expander), "notify::expanded",
		G_CALLBACK (toggled_expander_cb), rt);

	
	GtkSizeGroup *size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
//	e2_option_bool_add_dialog_widget (rt->dialog, vbox,
//		size_group, opt_show_last);
	e2_option_bool_add_dialog_widget (rt->dialog, rt->option_box,
		size_group, opt_case_sensitive);
//	e2_option_bool_add_dialog_widget (rt->dialog, vbox,
//		size_group, opt_search_backward);
	e2_option_bool_add_dialog_widget (rt->dialog, rt->option_box,
		size_group, opt_wrap);
		

	rt->info_box = e2_widget_get_box (TRUE, TRUE, TRUE);
	rt->info_expander = e2_option_bool_add_expander_widget (rt->dialog,
		vbox, _("info"), opt_show_info, rt->info_box);
	gtk_widget_show (rt->info_expander);
	g_signal_connect (G_OBJECT (rt->info_expander), "notify::expanded",
		G_CALLBACK (toggled_expander_cb), rt);
	
	GtkSizeGroup *group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
	GtkWidget *hbox = e2_widget_add_box (rt->info_box, FALSE, 0, FALSE, FALSE, 0);
	GtkWidget *label = _e2_widget_add_label (hbox,
		_("<small>result: </small>"), 0.0, 0.0, FALSE, 0);
	rt->info_label = e2_widget_add_label (hbox, "", 0.0, TRUE, 0);
	gtk_size_group_add_widget (group, label);
	hbox = e2_widget_add_box (rt->info_box, FALSE, 0, FALSE, FALSE, 0);
	gtk_widget_show (hbox);
	
	//now add some buttons
	_e2_dialog_add_button (rt->dialog, GTK_STOCK_FIND, "_find", 10);
	_e2_dialog_add_button (rt->dialog, GTK_STOCK_CLOSE, "_close", GTK_RESPONSE_CLOSE);
	e2_dialog_set_responses (rt->dialog, 10, GTK_RESPONSE_CLOSE);

	GtkWidget *menu = e2_option_create_menu (rt->dialog, NULL,
		opt_connected, e2_menu_control_cb, NULL,
		opt_show_last, toggled_cb1, rt,
		opt_case_sensitive, toggled_cb2, rt,
		opt_search_backward, toggled_cb3, rt,
		opt_wrap, toggled_cb4, rt,
		NULL);
	e2_option_connect (rt->dialog, _e2_option_bool_get (opt_connected));
	e2_dialog_attach_menu (rt->dialog, menu);

	g_signal_connect (G_OBJECT (rt->dialog), "show", G_CALLBACK (show_cb), rt);
	return rt->dialog;
}

void e2_search_dialog_show (GtkWidget *textview, gchar *arg, GtkWidget *parent)
{
	if ((parent == NULL) || (!GTK_IS_WINDOW (parent)))
		parent = app.main_window;
	if (textview == NULL)
		textview = GTK_WIDGET (app.output.text);
	GtkWidget *dialog = e2_search_dialog_create (textview);
	e2_dialog_show (dialog, parent, 0, E2_DIALOG_DONT_SHOW_ALL);
	gtk_window_present (GTK_WINDOW (dialog));
}

void e2_search_dialog_init ()
{
	e2_action_register ("dialog.search", E2_ACTION_TYPE_ITEM,
		e2_search_dialog_show, NULL, FALSE);

	opt_connected = e2_option_bool_register ("dialog-search-connected",
		"dialogs.search:general", _("dialog connected"),
		_("When active, the search dialog is connected to the option system "
			"and will replicate option changes to other connected mkdir dialogs. "
			"Otherwise option changes will be temporary to the current dialog."), NULL, TRUE);

	opt_show_options = e2_option_bool_register ("dialog-search-show-options",
		"dialogs.search:general", _("show search options"),
		_("Additional search options may be shown in the search dialog."),
		NULL, TRUE);
	
	opt_show_info = e2_option_bool_register ("dialog-search-show-info",
		"dialogs.search:general", _("show search result info"),
		_("Shall the info frame with the search result be shown?"),
		NULL, TRUE);

	opt_show_last = e2_option_bool_register ("dialog-search-show-last",
		"dialogs.search:general", _("show last search string"),
		_("When opening the search dialog or after searching, "
		"the last search string is shown in the entry field."),
		NULL, TRUE);

	opt_case_sensitive = e2_option_bool_register ("dialog-search-case-sensitive",
		"dialogs.search:general", _("case sensitive"),
		_("When opening the search dialog or after searching, "
		"the search will be case sensitive."),
		NULL, FALSE);

	opt_search_backward = e2_option_bool_register ("dialog-search-search-backward",
		"dialogs.search:general", _("search backward"),
		_("When opening the search dialog or after searching, "
		"the search will be backward from current."),
		NULL, FALSE);
	
	opt_wrap = e2_option_bool_register ("dialog-search-wrap",
		"dialogs.search:general", _("wrap around"),
		_("After reaching the end of the textview, wrap around and continue "
			"searching at the beginning."),
		NULL, FALSE);

	e2_cache_list_register ("search-history", &history);
}

