/* $Id: e2_view_dialog.c,v 1.11 2004/07/23 18:02:38 cairo Exp $
 * 
 * Copyright (C) 2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "emelfm2.h"
#include "e2_search_dialog.h"
#include "e2_view_dialog.h"

static E2_OptionSet *opt_view_width;
static E2_OptionSet *opt_view_height;
static E2_OptionSet *opt_view_wrap;


  /*****************/
 /***** utils *****/
/*****************/

static void dialog_destroy (E2_ViewDialogRuntime *rt)
{
	gtk_widget_destroy (rt->dialog);
	g_free (rt->filename);
	if (rt->idle_id != 0)
		g_source_remove (rt->idle_id);
	g_free (rt);
}


  /*********************/
 /***** callbacks *****/
/*********************/

static void response_cb (GtkWidget *dialog, gint response, E2_ViewDialogRuntime *rt)
{
	switch (response)
	{
		case 10:
			{
				rt->wrap = !rt->wrap;
				gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (rt->textview),
						rt->wrap ? GTK_WRAP_WORD: GTK_WRAP_NONE); 
			}
			break;
		case 11:
			e2_search_dialog_show (rt->textview, NULL, dialog);
			break;
		case 12:
			e2_view_dialog_refresh (rt->textview, rt->filename);
			break;
		default:
			dialog_destroy (rt);
			return;
	}
	gtk_widget_grab_focus (GTK_WIDGET (rt->textview));
}

  /******************/
 /***** public *****/
/******************/

gboolean e2_view_dialog_refresh (GtkWidget *textview, gchar *filename)
{
	gchar *cont;
	if ((g_file_get_contents (filename, &cont, NULL, NULL)) == FALSE)
	{
		printd (WARN, "could not open file '%s' for reading", filename);
		gchar *msg = g_strconcat ("could not open file '", filename, "' for reading", NULL);
		status_error (msg);
		g_free (msg);
		return FALSE;
	}
	GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
	
	if (!g_utf8_validate (cont, -1, NULL))
	{
		printd (DEBUG, "trying to convert '%s' to UTF-8", filename);
		GError *utf_error = NULL;
		gchar *utf = g_locale_to_utf8 (cont, -1, NULL, NULL, &utf_error);
		if (utf != NULL)
			gtk_text_buffer_set_text (GTK_TEXT_BUFFER (buffer), utf, -1);
		else
		{
			gtk_text_buffer_set_text (GTK_TEXT_BUFFER (buffer), utf_error->message, -1);
			g_error_free (utf_error);
		}
		g_free (utf);
	} else
	{
		printd (DEBUG, "'%s' is UTF-8 encoded", filename);
		gtk_text_buffer_set_text (GTK_TEXT_BUFFER (buffer), cont, -1);
	}
	g_free (cont);
	return TRUE;
}

GtkWidget *e2_view_dialog_file_create (gchar *filename)
{
	gchar *file;
	if (!g_path_is_absolute (filename))
		file = g_build_filename (curr_view->dir, filename, NULL);
	else
		file = g_strdup (filename);
	if (!g_file_test (file, G_FILE_TEST_EXISTS))
	{
		gchar *msg = g_strconcat ("cannot open '", file, "' for reading, because it does not exist", NULL);
		status_error (msg);
		g_free (msg);
		g_free (file);
		return NULL;
	}
	if (g_file_test (file, G_FILE_TEST_IS_DIR))
	{
		gchar *msg = g_strconcat ("cannot open '", file, "' for reading, because it is a directory", NULL);
		status_error (msg);
		g_free (msg);
		g_free (file);
		return NULL;
	}
	//init runtime object
	E2_ViewDialogRuntime *rt = g_malloc (sizeof (E2_ViewDialogRuntime));
	rt->filename = file;
	rt->wrap = _e2_option_bool_get (opt_view_wrap);
	rt->window_width = _e2_option_int_get (opt_view_width);
	rt->window_height = _e2_option_int_get (opt_view_height);
	rt->idle_id = 0;

	rt->dialog = e2_dialog_create (NULL, file, filename, response_cb, rt);
	//create the view
	GtkWidget *sw = e2_widget_add_sw (GTK_DIALOG (rt->dialog)->vbox,
			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC, TRUE, 5);
	rt->textview = gtk_text_view_new ();
	gtk_container_add (GTK_CONTAINER (sw), rt->textview);
	// set view defaults
	gtk_text_view_set_editable (GTK_TEXT_VIEW (rt->textview), FALSE);
	gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (rt->textview), FALSE);
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (rt->textview), rt->wrap); 
	gint char_width, char_height;
	e2_widget_set_font(rt->textview, e2_option_str_get ("output-font"),
				 &char_width, &char_height);
	// add the text
	e2_view_dialog_refresh (rt->textview, file);
	// add the buttons
	GtkWidget *wrap_button = _e2_dialog_add_check_button (rt->dialog, TRUE, "_wrap", 10);
	g_object_set_data (G_OBJECT (rt->dialog), "wrap", wrap_button);
	_e2_dialog_add_button (rt->dialog, GTK_STOCK_FIND, "_find", 11);
	_e2_dialog_add_button (rt->dialog, GTK_STOCK_REFRESH, "_refresh", 12);
	_e2_dialog_add_button (rt->dialog, GTK_STOCK_CLOSE, "_close", GTK_RESPONSE_CLOSE);

//	GtkWidget *menu = e2_option_create_menu (NULL,
//		opt_show_last, toggled_cb1, rt,
//		opt_case_sensitive, toggled_cb2, rt,
//		opt_view_wrap, view_wrap_cb, rt,
//		NULL);
//	e2_menu_connect (menu,FALSE);
//	e2_dialog_attach_menu (rt->dialog, menu);

	// display it
	gtk_window_resize (GTK_WINDOW (rt->dialog), char_width * rt->window_width,
				char_height *rt->window_height);
	return rt->dialog;
}

void e2_view_dialog_show (gchar *filename)
{
	GtkWidget *dialog = e2_view_dialog_file_create (filename);
	e2_dialog_show (dialog, app.main_window, 0, 0);
	gtk_window_present (GTK_WINDOW (dialog));
}

void e2_view_dialog_init ()
{
	e2_action_register ("dialog.view", E2_ACTION_TYPE_ITEM,
		e2_view_dialog_show, NULL, FALSE);

	opt_view_width = e2_option_int_register ("dialog-view-width",
		"dialogs.view:general", _("window width (characters)"),
		_("This is the default width of the text window in character. Please note "
		"that the window will always be at least as wide as the button area of the "
		"dialog."), NULL, 84, 20, 1000);

	opt_view_height = e2_option_int_register ("dialog-view-height",
		"dialogs.view:general", _("window height"),
		_("This is the default height of the text window."), NULL, 30, 10, 1000);

	opt_view_wrap = e2_option_bool_register ("dialog-view-wrap",
		"dialogs.view:general", _("wrap text to window"),
		_("The displayed text will be wrapped to the window width."),
		NULL, TRUE);
}
