/*
 * $Id: user_prompt.c,v 1.12 2004/04/26 22:37:51 cairo Exp $
 *
 * Copyright (C) 2003 tooar <tooar@gmx.net>
 * Copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <string.h>
#include "emelfm2.h"
#include "debug.h"
#include "e2_dialog.h"
#include "e2_button.h"

static GtkWidget *dialog;
static GtkWidget *entry;

static gboolean ok_cb (GtkWidget *widget, gchar **string)
{
	gchar *s;

	s = (gchar *) gtk_entry_get_text(GTK_ENTRY(entry));
	if (strlen(s) == 0)
	{
		status_message(_("Invalid input\n"));
		return TRUE;;
	}

	*string = g_strdup(s);

	gtk_widget_destroy(dialog);
	gtk_main_quit();
	return TRUE;
}

static gboolean cancel_cb(GtkWidget *widget, gchar **string)
{
	*string = NULL;
	gtk_widget_destroy(dialog);
	gtk_main_quit();
	return TRUE;
}
	
static gboolean key_press_cb (GtkWidget *widget, GdkEventKey *event, gchar **string)
{
	if (event->keyval == GDK_Escape)
	{
		cancel_cb(NULL, string);
		return TRUE;
	}
	return FALSE;
}

void create_user_prompt (gchar *prompt, gchar *init_text, gboolean select_text, gchar **string)
{
	GtkWidget *dialog_vbox;
	GtkWidget *action_area;
	GtkWidget *wid;

	dialog = gtk_dialog_new();
	dialog_vbox = GTK_DIALOG(dialog)->vbox;
	action_area = GTK_DIALOG(dialog)->action_area;
	gtk_container_set_border_width(GTK_CONTAINER(dialog_vbox), 5);
	gtk_box_set_spacing(GTK_BOX(dialog_vbox), 5);
	g_signal_connect(GTK_OBJECT(dialog), "delete_event",
		G_CALLBACK (cancel_cb), NULL);
	g_signal_connect(GTK_OBJECT(dialog), "key_press_event",
		G_CALLBACK(key_press_cb), string);

	wid = e2_widget_add_label (dialog_vbox, prompt, 0.0, FALSE, 5);
	gtk_label_set_justify(GTK_LABEL(wid), GTK_JUSTIFY_LEFT);

	entry = add_entry(dialog_vbox, init_text, FALSE, 0);
	if (select_text)
		gtk_editable_select_region(GTK_EDITABLE(entry), 0, -1);
	g_signal_connect(GTK_OBJECT(entry), "activate", 
		G_CALLBACK(ok_cb), string);
	gtk_widget_grab_focus(entry);

	add_button(action_area, _("Ok"), TRUE, 0, ok_cb, string);
	add_button(action_area, _("Cancel"), TRUE, 0, cancel_cb, string);

	gtk_window_set_title (GTK_WINDOW (dialog), "user prompt");
	e2_dialog_setup (dialog, app.main_window);
	gtk_widget_show(dialog);
	
}

GtkWidget *create_user_prompt2 (gchar *prompt, gchar *init_text,
	gboolean select_text, GtkWidget **entry)
{
	GtkWidget *dialog = get_dialog (app.main_window,
		GTK_MESSAGE_QUESTION, prompt);
			
	*entry = e2_widget_add_entry (GTK_DIALOG (dialog)->vbox, init_text, FALSE, select_text);

	e2_dialog_add_button (dialog, E2_BUTTON_CANCEL | E2_BUTTON_OK);
	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);	

	gtk_widget_grab_focus (*entry);
	return dialog;
}
