/*
 * $Id: e2_bookmark.c,v 1.31 2004/06/08 23:47:34 cairo Exp $
 * 
 * Copyright (C) 2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelfm2.
 * emelfm2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelfm2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "emelfm2.h"
#include "e2_bookmark.h"
#include "e2_toolbar.h"
#include "e2_option.h"
#include "e2_action.h"
#include "debug.h"
#include "e2_utils.h"
#include "e2_button.h"
#include "e2_dialog.h"
#include "e2_context_menu.h"

static E2_OptionSet *set;

  /*****************/
 /***** utils *****/
/*****************/

static void recreate_toolbars ()
{
	if (app.toolbar.has_bookmarks)
		_e2_toolbar_recreate (&app.toolbar);
	if (app.commandbar.has_bookmarks)
		_e2_toolbar_recreate (&app.commandbar);
	if (app.pane1.toolbar.has_bookmarks)
		_e2_toolbar_recreate (&app.pane1.toolbar);
	if (app.pane2.toolbar.has_bookmarks)
		_e2_toolbar_recreate (&app.pane2.toolbar);
}

static void __bookmark_delete_cb (GtkTreeIter *iter)
{
	_e2_option_tree_del (set, iter);
	recreate_toolbars ();
	if ((app.context_menu != NULL) && (GTK_IS_MENU (app.context_menu)))
		gtk_menu_popdown (GTK_MENU (app.context_menu));
}

static void _bookmark_delete_cb (GtkWidget *dialog, gint response, GtkTreeRowReference *ref)
{
	if (response == GTK_RESPONSE_YES)
	{
		if (ref != NULL)
		{
			GtkTreePath *path = gtk_tree_row_reference_get_path (ref);
			if (path != NULL)
			{
				GtkTreeIter iter;
				if ((gtk_tree_model_get_iter (set->ex.tree.model, &iter, path)) == TRUE)
					__bookmark_delete_cb (&iter);
			}
		}
	}
	gtk_widget_destroy (dialog);
}

  /*********************/
 /***** callbacks *****/
/*********************/

static void bookmark_delete_cb (GtkWidget *widget, GtkTreePath *path)
{
	GtkTreeIter iter;
	if ((gtk_tree_model_get_iter (set->ex.tree.model, &iter, path)) == TRUE)
	{
		gint children = gtk_tree_model_iter_n_children (set->ex.tree.model, &iter);
		if (e2_option_bool_get ("bookmarks-confirm-delete") ||
			((children > 0) && (e2_option_bool_get ("bookmarks-confirm-multi-delete"))))
		{
			gchar *name;
			gtk_tree_model_get (set->ex.tree.model, &iter, 0, &name, -1);
			
			gchar *question;
			if (children > 0)
				question = g_strdup_printf ("are you sure that you want to "
					"delete the bookmark '<b>%s</b>' and <b>%d %s</b>?", name, children,
					children == 1 ? "child" : "children");
			else
				question = g_strdup_printf ("are you sure that you want to "
					"delete the bookmark '<b>%s</b>'?", name);
			
		
			GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION, question, "confirm bookmark delete",
				_bookmark_delete_cb, gtk_tree_row_reference_new (set->ex.tree.model, path));
			e2_dialog_show (dialog, app.main_window,
				E2_BUTTON_NO | E2_BUTTON_YES, 0);
			g_free (question);
			g_free (name);
		} else
			__bookmark_delete_cb (&iter);
	}
}

gboolean e2_bookmark_action_cb (gpointer num, gchar *path)
{
	gint real_num = GPOINTER_TO_INT (num);
	printd (DEBUG, "e2_bookmark_action_cb (num:%d,path:%s)", real_num, path);
	switch (real_num)
	{
		case 1:
			e2_pane_change_dir (&app.pane1, path);
			if ((e2_option_bool_get ("bookmarks-focus-after-open")) &&
				(!GTK_WIDGET_HAS_FOCUS (app.pane1.focus_widget)))
				e2_panes_switch ();
			break;
		case 2:
			e2_pane_change_dir (&app.pane2, path);
			if ((e2_option_bool_get ("bookmarks-focus-after-open")) &&
				(!GTK_WIDGET_HAS_FOCUS (app.pane2.focus_widget)))
				e2_panes_switch ();
			break;
		default:
			e2_pane_change_dir (NULL, path);
			break;
	}
	return TRUE;
}

void e2_bookmark_add_cb (GtkWidget *widget, gchar *arg)
{
	gboolean before = FALSE;
	gboolean parent = FALSE;
	if ((arg != NULL) && (*arg != '\0'))
	{
		if (g_str_equal (arg, "top"))
			before = TRUE;
		if (g_str_equal (arg, "child"))
		{
			parent = TRUE;
			before = TRUE;
		}
	}
	
	gchar *utf = _STR2UTF (curr_view->dir);
	gchar *_basename = g_path_get_basename (utf);
	gchar *basename;
	if (strchr (_basename, '_'))
	{
		basename = e2_utils_str_replace (_basename, "_", "__");
		g_free (_basename);
	} else
		basename = _basename;

	void *options[4];
	options[0] = basename;
	options[1] = "";
	options[2] = utf;
	options[3] = utf;
	GtkTreeIter iter;
	GtkTreeIter iter2;
	GtkTreePath *path = NULL;
	if (widget != NULL)
		path = g_object_get_data (G_OBJECT (widget), "parent");
	if ((path != NULL) && ((gtk_tree_model_get_iter (set->ex.tree.model, &iter2, path)) == TRUE))
		_e2_option_tree_add (set, &iter, &iter2, !parent,
				before, 4, options);
	else
		//with no parent, the behaviour of before is vice versa
		_e2_option_tree_add (set, &iter, NULL, FALSE,
			!before, 4, options);

	recreate_toolbars ();

	g_free (utf);
	g_free (basename);
}

gboolean e2_bookmark_click_cb (GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	printd (DEBUG, "e2_bookmark_click_cb");
	switch (event->button)
	{
		case 2:	
		{
			gchar *path = g_object_get_data (G_OBJECT (widget), "bookmark-path");
			if ((path != NULL) && (e2_option_bool_get ("bookmarks-button2-other")))
			{
				e2_pane_change_dir (app.pane_other, path);
				if (e2_option_bool_get ("bookmarks-focus-after-open"))
					e2_panes_switch ();
				return TRUE;
			}
		}
			break;
		case 3:
		{
			printd (DEBUG, "bookmark menu");
			GtkWidget *menu = gtk_menu_new ();
			GtkWidget *menuitem;
			menuitem = add_menu_item2 (menu, "_edit", GTK_STOCK_PROPERTIES, configure_cb, "bookmarks");
			e2_menu_add_separator (menu);
			menuitem = add_menu_item2 (menu, "_add below", GTK_STOCK_ADD, e2_bookmark_add_cb, NULL);
			g_object_set_data_full (G_OBJECT (menuitem), "parent", data, NULL);
			menuitem = add_menu_item2 (menu, "add as _child", GTK_STOCK_ADD, e2_bookmark_add_cb, "child");
			g_object_set_data_full (G_OBJECT (menuitem), "parent", data, NULL);
			menuitem = add_menu_item2 (menu, "_delete", GTK_STOCK_DELETE, bookmark_delete_cb, data);
			e2_menu_popup (menu, event->button, event->time);
			return TRUE;
		}
			break;
		default:
			break;
	}
	return FALSE;
}

  /******************/
 /***** public *****/
/******************/

void e2_bookmark_clean ()
{
//	e2_action_unregister ("<bookmark>");
}

void e2_bookmark_init ()
{
	set = e2_option_tree_register ("bookmarks", "bookmarks", "bookmarks", NULL,
		NULL, NULL, E2_OPTION_TREE_UP_DOWN | E2_OPTION_TREE_ADD_DEL);
	e2_option_tree_add_column (set, "name", E2_OPTION_TREE_TYPE_STR, 0, "/",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, "icon", E2_OPTION_TREE_TYPE_ICON, 0, "",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, "tooltip", E2_OPTION_TREE_TYPE_STR, 0, "",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, "path", E2_OPTION_TREE_TYPE_STR, 0, "/",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_sync_columns (set);
	_e2_option_tree_append_default (set,
		"<bookmarks",
		"toolbar folder|||",
		"	\\<b>_home</b>|gtk-home|$(HOME)|~",
		"	down||downloads|/mnt/downloads",
		"	cdrom||/mnt/cdrom|/mnt/cdrom",
		"/||root|/",
		"mnt||/mnt/|/mnt/",
		"home||/home/|/home/",
		"usr||/usr/|/usr/",
		"usr/local||/usr/local/|/usr/local/",
		">", NULL);

	e2_option_bool_register ("bookmarks-button2-other", "bookmarks.misc:options",
		_("open bookmark in other pane on button2 click"),
		_("the middle mouse button will open the selected bookmark in the other "
			"file pane"),
		NULL, TRUE);
	e2_option_bool_register ("bookmarks-focus-after-open",
		"bookmarks.misc:options", _("focus file pane after opening a bookmark in it"),
		_("after opening a bookmark in the inactive file pane, the file pane will "
			"be focused"),
		NULL, TRUE);
	e2_option_bool_register ("bookmarks-confirm-delete",
		"bookmarks.misc:confirmation", _("confirm delete of single bookmarks"),
		_("pop up confirmation dialog before actually deleting any bookmark"),
		_("bookmarks-confirm-multi-delete"), FALSE);
	e2_option_bool_register ("bookmarks-confirm-multi-delete",
		"bookmarks.misc:confirmation", _("confirm delete of multiple bookmarks"),
		_("pop up confirmation dialog before actually deleting a bookmark that has "
			"children"),
		NULL, TRUE);

//	e2_action_register ("bookmark", E2_ACTION_TYPE_ITEM,
//		e2_bookmark_action_cb, GINT_TO_POINTER (0), TRUE);
	e2_action_register ("bookmark.add", E2_ACTION_TYPE_ITEM, e2_bookmark_add_cb,
		NULL, TRUE);
	_e2_action_register ("bookmark.<widget>", E2_ACTION_TYPE_BOOKMARKS,
		e2_bookmark_action_cb, GINT_TO_POINTER (0), TRUE, E2_ACTION_EXCLUDE_ACCEL,
		NULL);
	
}

