/* $Id: e2_main.c,v 1.43 2004/08/14 12:01:08 cairo Exp $
 *
 * Copyright (C) 2003-2004 tooar <tooar@gmx.net>
 * Copyright (C) 1999 Michael Clark.
 *
 * 9-22-2000: Modified by Aurelien Gateau
 *            Added code for show hidden toggle button
 *            
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

/**
 * @file src/e2_main.c 
 * @brief main function
 *
 * This file contains the main function and some local startup check helpers.
 **/

/*! \mainpage emelFM2 API
  *
	* \attention This API-documentation is under construction!\n Plans are to 
	* provide you with additional information to make it easier to read the 
	* code and hopefully inspire you to enhance it. For now you will not find 
	* much and maybe outdated information as emelFM2 changes quickly. 
	* This should change with version 0.1.0.
  *
  * \section intro introduction
  *
  * Welcome to emelFM2-doxygen. On this page you will find all important 
	* aspects of coding in emelFM2 covered. It should provide you with an 
	* adequat guidance to better understand the code and contains three parts:\n
	* \arg \ref conventions
	* \arg \ref interna
	* \arg \ref visible\n
	*
	*
	* \section conventions conventions
	*
	* emelFM2-code should follow some layout and naming conventions:
	* \arg Use tabs, not spaces, to indent code. (Recommended is a tab size of 4.)
	* \arg Stick to the same consistant coding style (vertically aligned braces; 
	* a space after if, while, functions, etc.; "//" C++ style comments):
	* \code
	* void routine (void)
	*	{
	*		if (function (a, b, c))
	*		{
	*			//do something
	*			x = a + 1;
	*			printf ("break lines after "
	*				"80 characters like this");
	*		}
	*	}
	* \endcode
	* \arg emelFM2's own functions start with e2_ 
	* \arg 'private' functions begin with _e2 - those should not be used 
	* from other parts of the programm
	* \arg emelFM2's own constants begin with E2_
	* \arg files coded for emelFM2 begin with e2_
	* \arg all functions not beginning with e2_ are either gtk_, g_ 
	* or old one's from emelFM. The latter ones will be replaced in the future.
	* \arg .h files can included with just \"\" even if they are not in the 
	* same directory
	* 
	* If you don't know doxygen well: try to click on everything you are 
	* interested. Even within the source documentation you will find lots 
	* of links. It's quite fun.
	* 
	* \section interna emelFM2 interna
	*
	* This section explains how emelFM2 works inside.
	* \arg \ref option_sys
	* \arg \ref cache
	* \arg \ref action_sys
	* \arg \ref virtual_fs
	* \arg \ref treeview
	* \arg \ref plugins
	*
	* \subsection option_sys the option system
	*
	* The option system is initialized in \ref e2_main.c. It is first 
	* loaded with some dummy values. After all other initializations are 
	* finished those dummy-data will be replaces with input from the 
	* 'config' file.
	*
	* \subsection cache the cache
	*
	* ToDo
	*
	* \subsection action_sys the action system
	*
	* ToDo
	*
	* \subsection virtual_fs the virtual filesystem
	*
	* Not implemented yet.
	*
	* \subsection treeview treeview
	*
	* ToDo
	*
	* \subsubsection cell_rend cell-renderers
	*
	* ToDo
	*
	* \subsection plugins plugins
	*
	* \attention due to the early stage of developement of emelFM2, 
	* the plugin api will probably change one or the other time. thus 
	* the following information is obsolete, although most parts are 
	* still true as of the time of writing this paragraph.
	*
	* Here is an outline of the Plugin Writing process.\n\n
	* <em>Step 1)</em> \code#include "../emelfm2.h"\endcode You need this to 
	* get access to all the functions and data structures of emelFM2\n\n
	* <em>Step 2)</em> Write the function that does what you want your plugin to 
	* do when activated. For example, if you want a plugin that prints 
	* "Hello World", then you would write this function.
	* \code void hello_world()
	* {  
	* 	status_message("Hello World\n");
	* }
	* \endcode The restricions for this function are that it must return 
	* void and can take no parameters.\n\n
	* <em> Step 3)</em> Write an init_plugin function something like this:
	* \code gint init_plugin(Plugin *p)
	* {
	* 	p->name = "Hello World";
	* 	p->description = "Prints \"Hello World\" on the output window.";
	* 	p->plugin_cb = hello_world; 
	* 					// This is whatever you named the function in Step 2
	*		return TRUE;
	*	}\endcode
	*	
	* This function \b must be called \c init_plugin, take a <tt>Plugin * </tt>argument, and 
  * return \c gint.\n\n
	* <em> Step 4) </em>If your plugin needs any cleaning up when it is unloaded you can
	* *optionally* implement an unload_plugin function as follows:
	* \code void unload_plugin(Plugin *p)
	* {
	*   status_message("I'm being unloaded\n");
	* }\endcode
	* This function \b must be called \c unload_plugin, take a <tt>Plugin *</tt> argument, 
	* and return \c void.
	*
	* \n\n
	* \section visible emelFM2 visible
	*
	* This section is about parts of emelFM2 you can see and which interact
	* with the user. The following items are covered:
	* \arg \ref panes
	* \arg \ref toolbar
	* \arg \ref commline
	* \arg \ref menu
	* 
	* \subsection panes the panes
	*
	* ToDo
	*
	* \subsection toolbar the toolbar
	*
	* ToDo
	*
	* \subsection commline the commandline
	*
	* ToDo
	*
	* \subsubsection output_panel the output panel
	*
	* ToDo
	*
	* \subsection menu the menu
	*
	* ToDo
	*
  */

#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include "emelfm2.h"
#include "build.h"
#include "debug.h"
#include "e2_cl_option.h"
#include "e2_option.h"
#include "e2_action.h"
#include "e2_toolbar.h"
#include "e2_upgrade.h"
#include "e2_bookmark.h"
#include "e2_command.h"
#include "e2_command_line.h"
#include "e2_window.h"
#include "e2_output.h"
#include "e2_dialog.h"
#include "e2_context_menu.h"

emelFM2 app;
Config cfg;
FileView *curr_view;
FileView *other_view;

Column all_columns[MAX_COLUMNS] = {
	{"filename", 180, FILENAME, TRUE, name_sort},
	{"size", 80, SIZE, TRUE, size_sort},
	{"permissions", 80, PERM, TRUE, perm_sort},
	{"owner", 60, OWNER, TRUE, user_sort},
	{"group", 60, GROUP, TRUE, group_sort},
	{"modified", 80, MODIFIED, TRUE, date_sort},
	{"accessed", 80, ACCESSED, FALSE, date_sort},
	{"changed", 80, CHANGED, FALSE, date_sort}
};

GdkColor CLIST_COLOR;

static gint update_status_bar (gpointer data)
{
	static gint last_selected_rows;
	static gint last_total_rows;
	gchar status_text[128];
	gint selected_rows, total_rows;

	if (curr_view->tagged != NULL)
		selected_rows = g_list_length(curr_view->tagged);
	else
		selected_rows = g_list_length(GTK_CLIST(curr_view->clist)->selection);
	total_rows = GTK_CLIST(curr_view->clist)->rows;

	if ((selected_rows != last_selected_rows) || (total_rows != last_total_rows))
	{
		last_selected_rows = selected_rows;
		last_total_rows = total_rows;
		if (e2_option_bool_get ("show-updir-entry")) total_rows--;
		g_snprintf(status_text, sizeof(status_text), _("%d of %d files selected"),
							 selected_rows, total_rows);
		gtk_label_set_text(GTK_LABEL(app.status_bar), status_text);
	}

	return TRUE;
}

static void add_plugin (gchar *filename, gboolean show_in_menu)
{
	Plugin *p;
	gchar *path;

	if ((g_file_test (PLUGINS_DIR, G_FILE_TEST_IS_DIR)) == TRUE)
		path = g_build_filename (PLUGINS_DIR, filename, NULL);
	else
	{
		printd (WARN, "plugin dir doesn't exist: %s", PLUGINS_DIR);
		return;
	}

	if ((p = load_plugin(path)) != NULL)
	{
		p->show_in_menu = show_in_menu;
		cfg.plugins = g_list_append (cfg.plugins, p);
	}
}

static void check_locale ()
{
	printd (DEBUG, "check_locale ()");
	gchar *locale = gtk_set_locale ();
	printd (NOTICE, "current locale is '%s'", locale);
	const gchar *broken =  g_getenv ("G_BROKEN_FILENAMES");
	if (broken != NULL && g_str_equal (locale, "C"))
	{
		printf (_("Your current locale is '%s'.\n"), locale);
		printf
		(
			_("You have set the environment variable G_BROKEN_FILENAMES. GTK+ is\n"
			"supposed to convert filenames from the system locale to UTF8 encoding.\n"
			"However, you have not set a system locale. Thus, please adjust your locale.\n")
		);
		if (!e2_cl_options.ignore_problems)
		{
			printf
			(
				_("(Note: There is the command line option -i/--ignore-problems, but use it\n"
				"at your own risk!)\n")
			);
			exit (1);
		} else
			printf
			(
				_("emelFM2 will ignore locale problems when reading filenames because\n"
				"--ignore-problems/-i has been set. This might result in segfaults and all\n"
				"kind of problems. You should really set a system locale with the\n"
				"LANG environment variable\n.")
			);

	}
#ifdef ENABLE_NLS
	printd (DEBUG, "setting locale dir to '%s'", LOCALE_DIR);
	bindtextdomain (BINNAME, LOCALE_DIR);
	textdomain (BINNAME);
#endif

}

//blind log handler to stop gtk warnings
static void blind_log_handler () { }

  /****************/
 /***** main *****/
/****************/

int main (int argc, char *argv[])
{
	gboolean first_run = TRUE;

	all_columns[0].title = _("Filename");
	all_columns[1].title = _("Size");
	all_columns[2].title = _("Permissions");
	all_columns[3].title = _("Owner");
	all_columns[4].title = _("Group");
	all_columns[5].title = _("Modified");
	all_columns[6].title = _("Accessed");
	all_columns[7].title = _("Changed");

	//needed by copy- and moveprogressbar
	g_thread_init (NULL);
//	gdk_threads_init ();

	//gtk initialization
	printd (DEBUG, "gtk init");
	gtk_init (&argc, &argv);
	
	e2_cl_option_process (argc, argv);
	check_locale ();

	if (e2_cl_options.surpress_gtk_output)
		g_log_set_handler ("Gtk", G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL
			| G_LOG_FLAG_RECURSION, blind_log_handler, NULL);

	//some local init stuff
	e2_option_init ();
	e2_cache_init ();
	e2_actions_init ();
	e2_keybinding_init ();
	e2_bookmark_init ();
	e2_command_init ();
	e2_output_init ();
	e2_dialog_init ();
	e2_context_menu_init ();
	
	e2_option_read_from_file (DEFAULT_CONFIG_FILE);
	gchar **values = e2_list_to_strv (e2_cl_options.option_overrides);
	e2_option_read_from_string (values);
	g_strfreev (values);
	
	printd (DEBUG, "read old config");
	if (!read_config_file())
	{
		/* Set Default values */
		app.left_view.show_hidden = FALSE;
		app.right_view.show_hidden = FALSE;
		cfg.window_width = 640;
		cfg.window_height = 480;
	}
	else
		first_run = FALSE;

	if (!read_filetypes_file())
	{
		/* Setup some default filetypes */
		add_filetype("<directory>", "cd@cd %%d/%%f,cd in other@!?file.open in other", "directories");
		add_filetype("<executable>", "run@./%f", "executables");
		add_filetype("jpeg,jpg,png,xpm,gif", "xv,gimp", _("Image Files"));
		add_filetype("c,cpp,h,pl,java", "Edit With Vi@x vi,Edit With Emacs@x emacs", _("Source Code Files"));
		add_filetype("o,so,a", "View Symbols@x nm %f | less", _("Object Files"));
		add_filetype("htm,html", "netscape,x lynx", _("HTML Documents"));
		add_filetype("tar", "PLUGIN:Unpack@!?Unpack,Unpack@tar xvf %f,Unpack in other panel@cd %D; tar xvf %d/%f,View Contents@tar tvf %f | less", _("Plain Tarballs"));
		add_filetype("tar.gz,tgz", "PLUGIN:Unpack@!?Unpack,Unpack@gunzip -c %f | tar xvf -,Unpack in other panel@cd %D; gunzip -c %d/%f | tar xvf -,View Contents@gunzip -c %f | tar tvf - | less", _("Gzip Tarballs"));
		add_filetype("tar.bz2","PLUGIN:Unpack@!?Unpack,Unpack@bzip2 -d -c %f | tar xvf -,Unpack in other panel@cd %D; bzip2 -d -c %d/%f | tar xvf -,View Contents@bzip2 -d -c %f | tar tvf - | less", _("Bzip2 Tarballs"));
		add_filetype("zip", "?Unpack,Unzip@unzip %f,Unzip in other panel@cd %D; unzip %d/%f,View Contents@unzip -l %f | less", _("ZIP Archives"));
		add_filetype("rpm", "View Info@rpm -qlip %f,Install@su \"rpm -Uvh \\\"%f\\\" \"", _("RPM Packages"));
		add_filetype("pdf", "acroread,xpdf", _("Adobe Acrobat Documents"));
		add_filetype("ps", "gv", _("PostScript Documents"));
		add_filetype("txt", "?View", _("Text Documents"));
		add_filetype("mp3,m3u,wav,ogg,mp2", "xmms,xmms -e", "audio files");
		add_filetype("avi,divx,mpg,mpeg,mov", "mplayer", "video files");
	}
	if (!read_plugins_file())
	{
		//Default Plugins
		add_plugin("glob.so", TRUE);
		add_plugin("rename_ext.so", TRUE);
		add_plugin("for_each.so", TRUE);
		add_plugin("clone.so", TRUE);
		add_plugin("pack.so", TRUE);
		add_plugin("copy_to_clipboard.so", TRUE);
		add_plugin("sort_by_ext.so", TRUE);
		add_plugin("du.so", TRUE);
		add_plugin("view.so", FALSE);
		add_plugin("unpack.so", FALSE);
	}

	//Build GUI
	printd (DEBUG, "create main window");
	e2_window_create (&app.window);
	printd (DEBUG, "init filters");
	initialize_filters (&app.right_view);
	initialize_filters (&app.left_view);

	if (e2_cl_options.pane1_path != NULL)
	{
		app.pane1.path = e2_cl_options.pane1_path;
		e2_pane_change_dir (&app.pane1, app.pane1.path);
	}
	
	if (e2_cl_options.pane2_path != NULL)
	{
		app.pane2.path = e2_cl_options.pane2_path;
		e2_pane_change_dir (&app.pane2, app.pane2.path);
	}

	gint pos = gtk_paned_get_position (GTK_PANED (app.window.output_paned));
	if (pos == 0)
		e2_action_run_simple ("command line: focus", NULL);
	
	e2_output_print_default (_("Type 'e2help' to get help on the command line!\n"));

	e2_upgrade_check (first_run);

	printd (DEBUG, "init auto-refresh");
	gtk_timeout_add (1000, update_status_bar, NULL);
	if (e2_option_bool_get ("auto-refresh-config"))
		cfg.check_config_id = gtk_timeout_add (1000, check_config_files, NULL);
	if (e2_option_bool_get ("auto-refresh"))
		cfg.auto_refresh_id = gtk_timeout_add (1000, auto_refresh, NULL);

	if (!e2_cl_options.no_detach && (e2_cl_options.debug_level < 5))
		daemon (1, 1);

	printd (DEBUG, "enter main loop");
	gtk_main ();

	exit (0);
}

