/* $Id: e2_pane.c,v 1.34 2004/06/04 15:37:58 cairo Exp $
 * 
 * Copyright (C) 2004 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include "emelfm2.h"
#include "e2_pane.h"
#include "e2_option.h"
#include "e2_cache.h"
#include "e2_action.h"
#include "e2_window.h"
#include "e2_utils.h"
#include "debug.h"

  /*****************/
 /***** utils *****/
/*****************/

static gboolean cd_checks (gchar *path)
{
	if ((g_file_test (path, G_FILE_TEST_EXISTS)) != TRUE)
	{
		gchar *message = g_strconcat ("The directory '", path, "' does not exist!\n", NULL);
		e2_output_print_error (message);
		g_free (message);
		return FALSE;
	}
	
	if ((g_file_test (path, G_FILE_TEST_IS_DIR)) != TRUE)
	{
		gchar *message = g_strconcat ("The file '", path, "' is not a directory!\n", NULL);
		e2_output_print_error (message);
		g_free (message);
		return FALSE;
	}
	
	if ((g_file_test (path, G_FILE_TEST_IS_EXECUTABLE)) != TRUE)
	{
		gchar *message = g_strconcat ("No permission to change to directory '", path, "'!\n", NULL);
		e2_output_print_error (message);
		g_free (message);
		return FALSE;
	}
	return TRUE;
}

  /*********************/
 /***** callbacks *****/
/*********************/

static gboolean focus_check_cb (GtkWidget *widget, GdkEventButton *event, E2_PaneRuntime *rt)
{
	printd (DEBUG, "focus_check_cb (widget:,event:,rt:%s)", rt->name);
	if (rt != app.pane_current)
		e2_panes_switch ();
	else if (!GTK_WIDGET_HAS_FOCUS (rt->focus_widget))
		gtk_widget_grab_focus (rt->focus_widget);
	return FALSE;
}

static void focus_check_cb2 (GtkWidget *widget, E2_PaneRuntime *rt)
{
	printd (DEBUG, "focus_check_cb2 (widget:,rt:%s)", rt->name);
	if (rt != app.pane_current)
		e2_panes_switch ();
}

  /*******************/
 /***** actions *****/
/*******************/

void e2_panes_switch ()
{
	printd (DEBUG, "e2_panes_switch ()");
	E2_PaneRuntime *tmp = app.pane_other;
	app.pane_other = app.pane_current;
	app.pane_current = tmp;
	clist_select_rows (tmp->view->clist, tmp->view->old_selection);
	switch_views ();
	gtk_widget_grab_focus (tmp->focus_widget);
	e2_hook_list_run (&app.hook_pane_focus_changed, tmp);
}

void e2_pane_change_dir (E2_PaneRuntime *rt, gchar *path)
{
	_e2_pane_change_dir (rt, path, FALSE, FALSE);
}

void _e2_pane_change_dir (E2_PaneRuntime *rt, gchar *path, gboolean no_update,
	gboolean no_hook)
{
	printd (DEBUG, "_e2_pane_change_dir (rt:,path:%s,no_update:%d,no_hook:%d)",
		path, no_update, no_hook);

	//work on currently active file pane if argument is NULL
	if (rt == NULL)
		rt = app.pane_current;

	if ((path == NULL) || (*path == '\0'))
		return;

	path = g_strdup (path);
	
	g_strstrip (path);

	//quick exit
	if ((path == NULL) || (*path == '\0'))
	{
		g_free (path);
		return;
	}

	//home dir check
	if (path[0] == '~')
	{
		gchar *free = path;
		if (path[1] != '\0')
		{
			path = g_strconcat (g_get_home_dir (), path + 1, NULL);
		} else
			path = g_strdup (g_get_home_dir ());
		g_free (free);
	}

	//transparent link dirs check
	if (_e2_option_bool_get (rt->opt_transparent))
	{
		gchar *free = path;
		path = e2_translate_to_filename (rt->path, path);
		g_free (free);
	}

	chdir (rt->path);

	gchar *str = _UTF2STR (path);
	if (!cd_checks (str))
	{
		g_free (str);
		g_free (path);
		return;
	}
	
	if (chdir (str))
	{
		e2_output_print_errno ();
		g_free (str);
		g_free (path);
		return;
	}

	g_free (str);
	g_free (rt->path);
	if (_e2_option_bool_get (rt->opt_transparent))
	{
		rt->path = path;
		str = _UTF2STR (rt->path);
	} else
	{
		str = g_get_current_dir ();
		gchar *utf = _STR2UTF (str);
		rt->path = g_strconcat (utf, "/", NULL);
		g_free (utf);
	}
	
	gchar *old = g_strdup (rt->view->dir);
	g_strlcpy (rt->view->dir, str, PATH_MAX);
	change_dir (rt->view, str, old);
	g_free (old);
	g_free (str);
	
	if (!no_update)
	{
		if (rt->history_cur < g_list_length (rt->history))
		{
			GList *free = g_list_nth (rt->history, rt->history_cur);
			GList *last = g_list_nth (rt->history, rt->history_cur - 1);
			if (last == NULL)
			{
				rt->history = NULL;
			} else
				last->next = NULL;
			e2_list_free_with_data (&free);
		}
		e2_list_update_history (rt->path, &rt->history,
			&rt->history_cur, 30, TRUE);
	}
	if (!no_hook)
		e2_hook_list_run (&rt->hook_change_dir, rt->path);
}

void e2_pane_go_forward (E2_PaneRuntime *rt, gchar *number)
{
	if (rt == NULL)
		rt = app.pane_current;
	_e2_pane_go_forward (rt, 1);
}

void _e2_pane_go_forward (E2_PaneRuntime *rt, gint number)
{
	printd (DEBUG, "_e2_pane_go_forward (rt:,number:%d", number);
	gint old = rt->history_cur;
	gint len = g_list_length (rt->history);
	rt->history_cur += number;
	if (rt->history_cur > g_list_length (rt->history))
		rt->history_cur = len;
	if (old != rt->history_cur)
		_e2_pane_change_dir (rt, g_list_nth_data (rt->history, rt->history_cur - 1), TRUE, FALSE);
}

void e2_pane_go_back (E2_PaneRuntime *rt, gchar *number)
{
	if (rt == NULL)
		rt = app.pane_current;
	_e2_pane_go_back (rt, 1);
}

void _e2_pane_go_back (E2_PaneRuntime *rt, gint number)
{
	printd (DEBUG, "_e2_pane_go_back (rt:,number:%d", number);
	gint old = rt->history_cur;
	rt->history_cur -= number;
	if (rt->history_cur < 1)
		rt->history_cur = 1;
	if (old != rt->history_cur)
		_e2_pane_change_dir (rt, g_list_nth_data (rt->history, rt->history_cur - 1), TRUE, FALSE);
}

void e2_pane_go_up (E2_PaneRuntime *rt, gchar *number)
{
	e2_pane_change_dir (rt, "..");
}

static void focus_action (E2_PaneRuntime *rt, gpointer data)
{
	if (app.pane_current != rt)
		e2_panes_switch ();
}

  /******************/
 /***** public *****/
/******************/

void e2_pane_create (E2_PaneRuntime *rt, gchar *option, gchar *parent, gint num)
{
	rt->pane = NULL;
	rt->name = g_strdup_printf ("%s %d", option, num);
	e2_cache_str_register (rt->name, &rt->path);
	//register pane toolbar option
	gchar *option_name = g_strdup_printf ("%sbar %d", option, num);
	gchar *group_name = g_strconcat (parent, ".", option_name, NULL);
	rt->set = e2_option_tree_register (option_name, group_name, option_name, NULL,
		NULL, NULL, E2_OPTION_TREE_UP_DOWN | E2_OPTION_TREE_ADD_DEL);
	e2_option_tree_add_column (rt->set, "name", E2_OPTION_TREE_TYPE_STR, 0,
		"separator", NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (rt->set, "icon", E2_OPTION_TREE_TYPE_ICON, 0,
		"", NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (rt->set, "tooltip", E2_OPTION_TREE_TYPE_STR, 0,
		"", NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (rt->set, "action", E2_OPTION_TREE_TYPE_SEL, 0,
		"<separator>", e2_actions_fill_store,
		(void *) E2_ACTION_EXCLUDE_TOOLBAR, 0, NULL, NULL);
	e2_option_tree_add_column (rt->set, "argument", E2_OPTION_TREE_TYPE_STR, 0,
		"", NULL, NULL, 0, NULL, NULL);
	e2_option_tree_sync_columns (rt->set);
	//FIXME: make this really dynamic
	if (num == 1)
	{
		_e2_option_tree_append_default (rt->set,
			"<panebar 1",
			"toggle|gtk-goto-last|toggle panel size|panes: adjust ratio|*,100%",
			"dir line 1|||pane 1: <dir line>|,expand",
			"marks|gtk-help||<submenu>|",
			"	add bookmark|||bookmark: add|",
			"	add to top|||bookmark: add|top",
			"	edit bookmarks|||config: configure|bookmarks",
			"	separator|||<separator>|",
			"	bookmarks|||pane 1: <bookmarks>|",
			"back|gtk-go-back|go back|pane 1: go back|",
			"up|gtk-go-up|go up|pane 1: go up|",
			"forward|gtk-go-forward|go forward|pane 1: go forward|"
			">", NULL);
		
	} else if (num == 2)
	{
		_e2_option_tree_append_default (rt->set, 
			"<panebar 2",
			"back|gtk-go-back|go back|pane 2: go back|",
			"up|gtk-go-up|go up|pane 2: go up|",
			"forward|gtk-go-forward|go forward|pane 2: go forward|",
			"marks|gtk-help||<submenu>|",
			"	add bookmark|||bookmark: add|",
			"	add to top|||bookmark: add|top",
			"	edit bookmarks|||config: configure|bookmarks",
			"	separator|||<separator>|",
			"	bookmarks|||pane 2: <bookmarks>|",
			"dir line 2|||pane 2: <dir line>|,expand",
			"|gtk-goto-first||panes: adjust ratio|*,0",
			">", NULL);
	}
	if ((!rt->set->ex.tree.found_in_config_file)
		&& (rt->set->ex.tree.def != NULL))
			e2_option_read_from_string (rt->set->ex.tree.def);

	rt->set->should_be_freed = TRUE;

	rt->opt_transparent = e2_option_get ("transparent-dir-links");

	option_name = g_strconcat (rt->name, "-use-startup-dir", NULL);
	group_name = g_strconcat (parent, ":startup", NULL);
	gchar *desc = g_strconcat ("use ", rt->name, " startup directory", NULL);
	rt->opt_use_startup = e2_option_bool_register (option_name, group_name,
		desc, "the directory entered below will be used", NULL, FALSE);
	rt->opt_use_startup->should_be_freed = TRUE;
	option_name = g_strconcat (option_name, "-startup-dir", NULL);
	desc = g_strconcat (rt->name, " startup directory:", NULL);
	rt->opt_startup = e2_option_str_register (option_name, g_strdup (group_name),
		desc,  "sets a certain directory to load at startup, otherwise the last "
		"opened directory will be used", rt->opt_use_startup->name, "/");
	rt->opt_startup->should_be_freed = TRUE;

	//check if the user set a startup dir
	if (_e2_option_bool_get (rt->opt_use_startup))
		rt->path = g_strdup (_e2_option_str_get (rt->opt_startup));
	
	if (rt->path == NULL)
		rt->path = g_strdup ("/");

	gint toolbar_defaults[] =
		{ TRUE, TRUE, 2, 1 + num, TRUE, 0, FALSE, FALSE, 0, 1 };
	e2_toolbar_create (&rt->toolbar, rt->set->name, rt->set->group,
		toolbar_defaults);

	e2_action_register ("panes.go forward", E2_ACTION_TYPE_ITEM,
		e2_pane_go_forward, NULL, FALSE);
	e2_action_register ("panes.go back", E2_ACTION_TYPE_ITEM,
		e2_pane_go_back, NULL, FALSE);
	e2_action_register ("panes.go up", E2_ACTION_TYPE_ITEM,
		e2_pane_go_up, NULL, FALSE);
	gchar *action_name = g_strconcat (rt->name, ": focus", NULL);
	e2_action_register (action_name, E2_ACTION_TYPE_ITEM,
		focus_action, rt, TRUE);
	g_free (action_name);
	action_name = g_strconcat (rt->name, ".<dir line>", NULL);
	_e2_action_register (action_name, E2_ACTION_TYPE_COMMAND_LINE,
		e2_command_line_directory_cb, "dir line", FALSE,
		E2_ACTION_EXCLUDE_MENU | E2_ACTION_EXCLUDE_ACCEL, rt);
	g_free (action_name);
	action_name = g_strconcat (rt->name, ".go forward", NULL);
	e2_action_register (action_name, E2_ACTION_TYPE_ITEM,
		e2_pane_go_forward, rt, TRUE);
	g_free (action_name);
	action_name = g_strconcat (rt->name, ".go back", NULL);
	e2_action_register (action_name, E2_ACTION_TYPE_ITEM,
		e2_pane_go_back, rt, TRUE);
	g_free (action_name);
	action_name = g_strconcat (rt->name, ".go up", NULL);
	e2_action_register (action_name, E2_ACTION_TYPE_ITEM,
		e2_pane_go_up, rt, FALSE);
	g_free (action_name);
	action_name = g_strconcat (rt->name, ".<bookmarks>", NULL);
	_e2_action_register (action_name, E2_ACTION_TYPE_BOOKMARKS,
		e2_bookmark_action_cb, GINT_TO_POINTER (num), TRUE,
		E2_ACTION_EXCLUDE_ACCEL, NULL);
	g_free (action_name);
	
	_e2_pane_create (rt);
}

void _e2_pane_create (E2_PaneRuntime *rt)
{
	g_hook_list_init (&rt->hook_change_dir, sizeof (GHook));
	
	gchar *history_name = g_strconcat (rt->name, "-history", NULL);
	e2_cache_list_register (history_name, &rt->history);
	if (rt->history != NULL)
		rt->history_cur = g_list_length (rt->history);
	g_free (history_name);

	rt->pane = create_file_view (rt->view);
	rt->focus_widget = rt->view->clist;
	rt->outer_box = gtk_hbox_new (FALSE, 0);
	rt->inner_box = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start_defaults (GTK_BOX (rt->inner_box), rt->pane);
	gtk_box_pack_start_defaults (GTK_BOX (rt->outer_box), rt->inner_box);
	
	g_signal_connect (G_OBJECT (rt->focus_widget), "button_press_event", G_CALLBACK (focus_check_cb), rt);
	g_signal_connect (G_OBJECT (rt->focus_widget), "grab-focus", G_CALLBACK (focus_check_cb2), rt);
	e2_keybinding_register ("main.file pane", rt->focus_widget);
		
	gtk_widget_show (rt->inner_box);
	gtk_widget_show (rt->outer_box);

	_e2_toolbar_create (&rt->toolbar);
}

void e2_pane_destroy (E2_PaneRuntime *rt)
{
	gchar *history_name = g_strconcat (rt->name, "-history", NULL);
	e2_cache_list_unregister (history_name);
	g_free (history_name);
	e2_toolbar_destroy (&rt->toolbar);
	gtk_widget_destroy (rt->pane);
	g_hook_list_clear (&rt->hook_change_dir);
}

void e2_pane_recreate (E2_PaneRuntime *rt)
{
	e2_pane_destroy (rt);
	_e2_pane_create (rt);
}

