/* $Id: e2_window.c,v 1.36 2004/07/23 18:02:38 cairo Exp $
 * 
 * Copyright (C) 2004 tooar <tooar@gmx.net>
 * Copyright (C) 1999 Michael Clark.
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file COPYING. If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "emelfm2.h"
#include "e2_option.h"
#include "e2_action.h"
#include "e2_context_menu.h"
#include "e2_toolbar.h"
#include "e2_utils.h"
#include "build.h"
#include "e2_window.h"
#include "e2_output.h"
#include "e2_command_line.h"

/**
 * @file src/e2_window.c 
 * @brief main window functions
 *
 * main window functions
 *
 **/

static gdouble get_pos (GtkWidget *paned);

  /*********************/
 /***** callbacks *****/
/*********************/

static gboolean panes_adjusted_cb (GtkWidget *widget, GtkAllocation *alloc, E2_WindowRuntime *rt)
{
//	printd (DEBUG, "panes_adjusted_cb (widget:,alloc:%d-%d,rt:_)", alloc->width, alloc->height);
	rt->panes_paned_ratio = get_pos (rt->panes_paned);
//	printd (DEBUG, "1ratio: %f", rt->panes_paned_ratio);
	if (((rt->panes_paned_ratio == 0.0) && (app.pane_current == &app.pane1)) ||
		((rt->panes_paned_ratio == 1.0) && (app.pane_current == &app.pane2)))
		e2_panes_switch ();
	if ((rt->panes_paned_ratio != 0.0) && (rt->panes_paned_ratio != 1.0))
		rt->panes_paned_ratio_last = rt->panes_paned_ratio;
	return TRUE;
}

static gboolean panes_adjusted_cb2 (GtkWidget *widget, GtkAllocation *alloc, E2_WindowRuntime *rt)
{
//	printd (DEBUG, "panes_adjusted_cb2 (widget:,alloc:%d-%d,rt:_)", alloc->width, alloc->height);
	rt->output_paned_ratio = get_pos (app.window.output_paned);
//	printd (DEBUG, "2ratio: %f", rt->output_paned_ratio);
	if ((rt->output_paned_ratio != 0.0) && (rt->output_paned_ratio != 1.0))
		rt->output_paned_ratio_last = rt->output_paned_ratio;
	return TRUE;
}

static void show_cb (GtkWidget *window, E2_WindowRuntime *rt)
{
	printd (NOTICE, "showing main window");
	//set position again
	gtk_paned_set_position (GTK_PANED (rt->panes_paned),
		GTK_PANED (rt->panes_paned)->max_position * rt->panes_paned_ratio);
	gtk_paned_set_position (GTK_PANED (app.window.output_paned),
		GTK_PANED (app.window.output_paned)->max_position * rt->output_paned_ratio);
	rt->panes_paned_pos = gtk_paned_get_position (GTK_PANED (rt->panes_paned));
	rt->output_paned_pos = gtk_paned_get_position (GTK_PANED (app.window.output_paned));

	g_signal_connect (G_OBJECT (app.pane1.outer_box), "size-allocate",
		G_CALLBACK (panes_adjusted_cb), rt);
	g_signal_connect (G_OBJECT (app.output.text), "size-allocate",
		G_CALLBACK (panes_adjusted_cb2), rt);
	WAIT_FOR_EVENTS;
	GList *tmp;
	for (tmp = e2_cl_options.startup_commands; tmp != NULL; tmp = g_list_next (tmp))
	{
		e2_command_run (tmp->data, E2_COMMAND_RANGE_DEFAULT);
	}
}

  /*****************/
 /***** utils *****/
/*****************/

static void create_pane_boxes (E2_WindowRuntime *rt)
{
	if (e2_option_bool_get ("panes-horizontal"))
	{
		rt->panes_horizontal = TRUE;
		rt->panes_outer_box = gtk_hbox_new (FALSE, 0);
		rt->panes_inner_box = gtk_vbox_new (FALSE, 0);
		rt->panes_paned = gtk_vpaned_new ();
	}
	else
	{
		rt->panes_horizontal = FALSE;
		rt->panes_outer_box = gtk_vbox_new (FALSE, 0);
		rt->panes_inner_box = gtk_hbox_new (FALSE, 0);
		rt->panes_paned = gtk_hpaned_new ();
	}
	gtk_box_pack_start_defaults (GTK_BOX (rt->panes_outer_box), rt->panes_inner_box);
	gtk_box_pack_start_defaults (GTK_BOX (rt->panes_inner_box), rt->panes_paned);

	gtk_widget_show (rt->panes_outer_box);
	gtk_widget_show (rt->panes_inner_box);
	gtk_widget_show (rt->panes_paned);
}

static gdouble get_pos (GtkWidget *paned)
{
	GtkPaned *_paned = GTK_PANED (paned);
	gint pos = gtk_paned_get_position (_paned);
	return ((gdouble) pos / _paned->max_position);
}

static gint get_pos_from_str (gchar *str, GtkPaned *paned)
{
	if (str == NULL)
		return -2;
	
	gboolean percent = FALSE;
	if (g_str_has_suffix (str, "%"))
		percent = TRUE;

	gchar *end = NULL;
	gdouble num = g_ascii_strtod (str, &end);

	if (end == str)
		return -2;
	
	gint retval;
	if (percent)
		retval = paned->max_position * (num / 100.0);
	else
		retval = num;
	
	return retval;
}

  /*******************/
 /***** actions *****/
/*******************/

static void toggle_direction_action (GtkWidget *widget)
{
	e2_option_bool_toggle ("panes-horizontal");
	e2_window_recreate (&app.window);
}

static void _adjust_ratio_action (GtkPaned *paned, gint *pos,
	gdouble *ratio_last, gchar *arg)
{
	printd (DEBUG, "_adjust_ratio_action (paned:,pos:%d,ratio_last:%f,arg:%s)",
		*pos, *ratio_last, arg);

	if (arg == NULL)
		return;

	gint cur = -2;

	gchar *second = g_utf8_strchr (arg, -1, ',');
	if (second != NULL)
	{
		glong offset =  g_utf8_pointer_to_offset (arg, second);
		gchar *temp_str = e2_utf8_ndup (arg, offset);
		gint one = get_pos_from_str (temp_str, paned);
		g_free (temp_str);
		gint two = get_pos_from_str (second + 1, paned);
		if (two == -2)
			return;
		cur = gtk_paned_get_position (paned);
		if (cur == two)
		{
			if (one == -2)
			{
				if ((*pos == paned->max_position) || (*pos == paned->min_position))
					cur = paned->max_position * *ratio_last;
				else
					cur = *pos;
			}
			else
				cur = one;
		}
		else
		{
			if ((cur != 0) && (cur != paned->max_position))
				*pos = cur;
			cur = two;
		}
	} else
	{
		cur = get_pos_from_str (arg, paned);
		if (cur == -2)
		{
			if ((*pos == paned->max_position) || (*pos == paned->min_position))
				cur = paned->max_position * *ratio_last;
			else
				return;
//				cur = *pos;
		} else
			*pos = cur;
	}
	gtk_paned_set_position (paned, cur);
}

static void adjust_ratio_action1 (E2_WindowRuntime *rt, gchar *arg)
{
	_adjust_ratio_action (GTK_PANED (rt->panes_paned), &rt->panes_paned_pos,
		&rt->panes_paned_ratio_last, arg);
}

void adjust_ratio_action2 (E2_WindowRuntime *rt, gchar *arg)
{
	_adjust_ratio_action (GTK_PANED (rt->output_paned), &rt->output_paned_pos,
		&rt->output_paned_ratio_last, arg);
}

  /******************/
 /***** public *****/
/******************/

/**
 * @brief set default window icon list
 *
 * set default window icon list from files starting with @a base in @a icon_dir
 *
 * @param icon_dir directory where the icons are in
 * @param base filename base of the icon files
 *
 * @return 
 **/
void e2_window_set_icon (const gchar *icon_dir, const gchar *base)
{
	GError *error = NULL;
	GDir *dir = g_dir_open (icon_dir, 0, &error);
	if (error != NULL)
	{
		printd (WARN, "could not open window icon directory '%s' (%s)", icon_dir, error->message);
		g_error_free (error);
		return;
	}
		
	GList *list = NULL;
	const gchar *name;
	gint len = strlen (base) + 7;
	while ((name = g_dir_read_name (dir)) != NULL)
	{
		if (g_str_has_prefix (name, base) && g_str_has_suffix (name, ".png") &&
			strlen (name) == len)
		{
			gchar *file = g_build_filename (icon_dir, name, NULL);
			GError *error = NULL;
			GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file (file, &error);
			if (error != NULL)
			{
				printd (WARN, "could not open image file '%s' (%s)", file, error->message);
				g_free (file);
				g_error_free (error);
				continue;
			}
			g_free (file);
			list = g_list_append (list, pixbuf);
		}
	}
	gtk_window_set_default_icon_list (list);
}

/**
 * @brief create main window
 *
 * create main window
 *
 * @return 
 **/
void e2_window_create (E2_WindowRuntime *rt)
{
	e2_action_register ("panes.adjust ratio", E2_ACTION_TYPE_ITEM,
		adjust_ratio_action1, rt, TRUE);
	e2_action_register ("panes.refresh", E2_ACTION_TYPE_ITEM,
		refresh_cb, NULL, FALSE);
	e2_action_register ("panes.sync", E2_ACTION_TYPE_ITEM,
		sync_dirs_cb, NULL, FALSE);
	e2_action_register ("panes.toggle direction", E2_ACTION_TYPE_ITEM,
		toggle_direction_action, NULL, FALSE);
	e2_action_register ("panes.switch", E2_ACTION_TYPE_ITEM,
		e2_panes_switch, NULL, FALSE);

	e2_action_register ("output.adjust ratio", E2_ACTION_TYPE_ITEM,
		adjust_ratio_action2, rt, TRUE);

	g_hook_list_init (&app.hook_pane_focus_changed, sizeof (GHook));
	e2_update_settings ();
	e2_window_set_icon (ICON_DIR, "icon_emelfm2_");
	
	app.main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	//setup window
	gtk_window_set_role (GTK_WINDOW (app.main_window), "main");
	gtk_window_set_title (GTK_WINDOW (app.main_window), PROGNAME);
	gtk_window_set_wmclass (GTK_WINDOW (app.main_window), "main", BINNAME);
	gtk_widget_set_size_request (app.main_window, 0, 0);
	gtk_window_set_default_size (GTK_WINDOW (app.main_window), cfg.window_width, cfg.window_height);
	gtk_window_set_resizable (GTK_WINDOW (app.main_window), TRUE);
	
	g_signal_connect (GTK_OBJECT (app.main_window), "delete_event",
		G_CALLBACK (quit_cb), NULL);

	e2_keybinding_register ("main", app.main_window);
	
	app.tooltips = gtk_tooltips_new ();

	app.hbox_main = gtk_hbox_new (FALSE, 0);
	app.vbox_main = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (app.main_window), app.hbox_main);
	gtk_box_pack_start_defaults (GTK_BOX (app.hbox_main), app.vbox_main);
	gtk_widget_show (app.hbox_main);
	gtk_widget_show (app.vbox_main);

	create_pane_boxes (rt);
	
	app.pane1.view = &app.left_view;
	app.pane2.view = &app.right_view;
	e2_pane_create (&app.pane1, "pane", "panes", 1);
	e2_pane_create (&app.pane2, "pane", "panes", 2);
	app.pane_current = &app.pane1;
	app.pane_other = &app.pane2;
	curr_view = app.pane1.view;
	other_view = app.pane2.view;
	gtk_paned_pack1 (GTK_PANED (rt->panes_paned), app.pane1.outer_box, TRUE, TRUE);
	gtk_paned_pack2 (GTK_PANED (rt->panes_paned), app.pane2.outer_box, TRUE, TRUE);
	
	e2_cache_double_register ("file-pane-ratio-last", &rt->panes_paned_ratio_last, 0.55);
	e2_cache_double_register ("file-pane-ratio", &rt->panes_paned_ratio, 0.55);
	e2_cache_double_register ("output-pane-ratio-last", &rt->output_paned_ratio_last, 0.7);
	e2_cache_double_register ("output-pane-ratio", &rt->output_paned_ratio, 0.7);

	//Command Panel
	GtkWidget *wid = e2_output_create (&app.output);
	rt->output_paned = gtk_vpaned_new ();
	GTK_WIDGET_UNSET_FLAGS (rt->output_paned, GTK_CAN_FOCUS);
	gtk_paned_pack1(GTK_PANED (rt->output_paned), rt->panes_outer_box, TRUE, TRUE);
	gtk_paned_pack2(GTK_PANED (rt->output_paned), wid, TRUE, TRUE);
	gtk_widget_show (rt->output_paned);
	gtk_box_pack_start (GTK_BOX (app.vbox_main), rt->output_paned, TRUE, TRUE, 0);

	//staus bar
	app.status_bar = gtk_label_new ("ready");
	gtk_misc_set_alignment (GTK_MISC (app.status_bar), 0.5, 0.5);
	gtk_box_pack_start (GTK_BOX (app.vbox_main), app.status_bar, FALSE, TRUE, 0);
	gtk_widget_show (app.status_bar);

	gint toolbar_defaults[] =
		{ TRUE, TRUE, 2, 2, FALSE, 1, FALSE, TRUE, 3, 1 };
	e2_toolbar_create (&app.toolbar, "toolbar", "toolbar",
		toolbar_defaults);
	_e2_toolbar_create (&app.toolbar);
	gint commandbar_defaults[] =
		{ TRUE, TRUE, 2, 0, TRUE, 1, FALSE, FALSE, 3, 1 };
	e2_toolbar_create (&app.commandbar, "commandbar", "commandbar",
		commandbar_defaults);
	_e2_toolbar_create (&app.commandbar);
	
	//FIXME: lots of overhead
	_e2_toolbar_recreate (&app.commandbar);
	_e2_toolbar_recreate (&app.toolbar);

	//FIXME:
	//this is to prevent the "jumping" of the GtkPaned sliders
	//at startup, set position has to be called in show_cb again;
	//this adds some unnecessary overhead
	if (!rt->panes_horizontal)
	{
		gtk_widget_realize (rt->panes_paned);
		gtk_paned_set_position (GTK_PANED (rt->panes_paned),
			GTK_PANED (rt->panes_paned)->max_position * rt->panes_paned_ratio);
		gtk_widget_realize (rt->output_paned);
		gtk_paned_set_position (GTK_PANED (rt->output_paned),
			GTK_PANED (rt->output_paned)->max_position * rt->output_paned_ratio);
	}
	g_signal_connect (G_OBJECT (app.main_window), "show", G_CALLBACK (show_cb),
		rt);
	gtk_widget_show_all (app.main_window);


	e2_pane_change_dir (&app.pane1, app.pane1.path);
	e2_pane_change_dir (&app.pane2, app.pane2.path);
}

void e2_window_recreate (E2_WindowRuntime *rt)
{
	printd (DEBUG, "recreate main window");
	FileView *old_curr_view, *old_other_view;
	gfloat curr_scroll_pos, other_scroll_pos;
	GList *curr_selection, *other_selection;
	GtkAdjustment *vadj;
	
	//Save some info on the current view so we can reset it when done
	old_curr_view = curr_view;
	old_other_view = other_view;
	curr_scroll_pos = gtk_clist_get_vadjustment (GTK_CLIST (curr_view->clist))->value;
	other_scroll_pos = gtk_clist_get_vadjustment (GTK_CLIST (other_view->clist))->value;
	curr_selection = g_list_copy (GTK_CLIST (curr_view->clist)->selection);
	other_selection = g_list_copy (other_view->old_selection);

	//destroy the widgets
	e2_pane_destroy (&app.pane1);
	e2_pane_destroy (&app.pane2);
	e2_toolbar_destroy (&app.toolbar);
	e2_toolbar_destroy (&app.commandbar);
	gtk_widget_destroy (rt->panes_outer_box);

	e2_update_settings ();

	e2_keybinding_register ("main", app.main_window);

	create_pane_boxes (rt);
	
	_e2_pane_create (&app.pane1);
	_e2_pane_create (&app.pane2);
	gtk_paned_pack1 (GTK_PANED (rt->panes_paned), app.pane1.outer_box, TRUE, TRUE);
	gtk_paned_pack2 (GTK_PANED (rt->panes_paned), app.pane2.outer_box, TRUE, TRUE);
	g_signal_connect (G_OBJECT (app.pane1.outer_box), "size-allocate",
		G_CALLBACK (panes_adjusted_cb), rt);
	g_signal_connect (G_OBJECT (app.output.text), "size-allocate",
		G_CALLBACK (panes_adjusted_cb2), rt);

	gtk_paned_pack1(GTK_PANED (rt->output_paned), rt->panes_outer_box, TRUE, TRUE);
	
	_e2_toolbar_create (&app.commandbar);
	_e2_toolbar_create (&app.toolbar);

	e2_alias_sync (&app.aliases);

	e2_pane_change_dir (&app.pane1, app.pane1.path);
	e2_pane_change_dir (&app.pane2, app.pane2.path);
	
	e2_output_update_style (&app.output);
	
	gtk_clist_unselect_all (GTK_CLIST (old_curr_view->clist));
	clist_select_rows (old_curr_view->clist, curr_selection);
	e2_panes_switch ();
	g_list_free (curr_selection);
	
	gtk_clist_unselect_all (GTK_CLIST (old_other_view->clist));
	clist_select_rows (old_other_view->clist, other_selection);
	e2_panes_switch ();
	g_list_free (other_selection);
	
	vadj = gtk_clist_get_vadjustment (GTK_CLIST (old_curr_view->clist));
	gtk_adjustment_set_value (GTK_ADJUSTMENT (vadj), curr_scroll_pos);
	vadj = gtk_clist_get_vadjustment (GTK_CLIST (old_other_view->clist));
	gtk_adjustment_set_value (GTK_ADJUSTMENT (vadj), other_scroll_pos);

}
