/*
 *
 * emelFM
 * Copyright (C) 1999 Michael Clark.
 *
 * emelFM2 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

/* 9-22-2000: Modified by Aurelien Gateau 
 *            Added code for show hidden toggle button
 */

/* 10-9-2000: Modified by Aurelien Gateau 
 *            Added code for windows right click behavior
 */

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "emelfm2.h"
#include "debug.h"
#include "e2_context_menu.h"
#include "e2_option.h"
#include "e2_utils.h"
#include "e2_command.h"
#include "e2_output.h"

//#include "icons/icon_dirparent.xpm"

/* Stuff for drag and drop */
enum { TARGET_URL, TARGET_STRING };

static GtkTargetEntry target_table[] = {
	{ "text/uri-list", 0, TARGET_URL },
	{ "text/plain",    0, TARGET_STRING }
};
static guint n_targets = sizeof(target_table) / sizeof(target_table[0]);
static gboolean button2updir;
static gboolean updir_started;

void fileview_select_all ()
{
	gtk_clist_select_all (GTK_CLIST (curr_view->clist));
	if (e2_option_bool_get ("show-updir-entry")
		&& !g_str_equal (curr_view->dir, "/"))
			gtk_clist_unselect_row (GTK_CLIST (curr_view->clist), 0, 0);
}

void fileview_select_all2 (GtkWidget *blah, FileView *view)
{
	if (curr_view != view)
		e2_panes_switch ();
	gtk_clist_select_all (GTK_CLIST (curr_view->clist));
	if (e2_option_bool_get ("show-updir-entry")
		&& !g_str_equal (curr_view->dir, "/"))
		gtk_clist_unselect_row (GTK_CLIST (curr_view->clist), 0, 0);
}

/* This function is called whenever we need to exchange curr_view and
 * other_view. */
void switch_views ()
{
	printd (DEBUG, "switch views");
	gint i;
	gint num_cols;
	GtkStyle *style;
	FileView *temp;
	GList *tmp;

	temp = other_view;
	other_view = curr_view;
	curr_view = temp;

	if (other_view->old_selection)
		g_list_free(other_view->old_selection);
	other_view->old_selection = g_list_copy(
		    GTK_CLIST(other_view->clist)->selection);
	for (tmp = other_view->old_selection; tmp != NULL; tmp = tmp->next)
		gtk_clist_set_background(GTK_CLIST(other_view->clist), (gint)tmp->data,
			e2_option_color_get ("color-inactive-selected"));
	for (tmp = curr_view->old_selection; tmp != NULL; tmp = tmp->next)
		gtk_clist_set_background(GTK_CLIST(curr_view->clist), (gint)tmp->data,
			&CLIST_COLOR);

	gtk_clist_unselect_all(GTK_CLIST(other_view->clist));
	style = gtk_style_copy(gtk_widget_get_style(
		 GTK_CLIST(other_view->clist)->column[0].button));
	style->bg[GTK_STATE_NORMAL] = *e2_option_color_get ("color-active-pane");
	num_cols = GTK_CLIST(curr_view->clist)->columns;
	for (i = 0; i < num_cols; i++)
	{
		gtk_widget_set_style(GTK_CLIST(curr_view->clist)->column[i].button,
		 style);
		gtk_widget_set_style(curr_view->sort_arrows[i], style);
	}
	for (i = 0; i < num_cols; i++)
	{
		gtk_widget_restore_default_style(
		GTK_CLIST(other_view->clist)->column[i].button);
		gtk_widget_restore_default_style(other_view->sort_arrows[i]);
	}
}

gboolean toggle_show_hidden (FileView *view)
{
	if (view == NULL)
		view = curr_view;
	view->show_hidden = !view->show_hidden;
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (view->hidden_button),
		view->show_hidden);
	refresh_list(view);
	return TRUE;
}

gboolean toggle_show_hidden_cb(GtkWidget *widget, FileView *view)
{
	printd (DEBUG, "callback: toggle show hidden");
	view->show_hidden = GTK_CHECK_MENU_ITEM (widget)->active;
	refresh_list(view);
	return TRUE;
}

static gboolean mouse_release_cb (GtkWidget *widget, GdkEventButton *event, FileView *view)
{
	printd (DEBUG, "release");
	if (updir_started)
	{
		printd (DEBUG, "up");
		e2_pane_change_dir (NULL, "..");
		updir_started = FALSE;
	}
	return FALSE;
}

static gboolean mouse_click_cb (GtkWidget *widget, GdkEventButton *event, FileView *view)
{
	printd (DEBUG, "callback: mouse click");
	if (curr_view != view)
		e2_panes_switch ();
	if (event->button == 2)
	{
		// Middle Button: Drag and Drop
		printd (DEBUG, "button 2");
		if (button2updir)
			updir_started = TRUE;
		else
		{
			GtkTargetList *list;
			GdkDragContext *context;

			list = gtk_target_list_new(target_table, n_targets);
			context = gtk_drag_begin(curr_view->clist, list, GDK_ACTION_COPY |
			GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK,
			event->button, (GdkEvent *)event);
			gtk_drag_set_icon_default(context);
		}
		return TRUE;
	} else if (event->button == 3)
	{
		//context menu
		if (e2_option_bool_get ("windows-right-click") &&
			(view->tagged == NULL))
		{
			//if windows right click is enabled, we want to select the file 
			gint xmouse,ymouse,row,column;
			GdkModifierType mask;
			gdk_window_get_pointer (GTK_CLIST (view->clist)->clist_window,
				 &xmouse, &ymouse, &mask);
			if ((gtk_clist_get_selection_info (GTK_CLIST (view->clist), xmouse, ymouse,
		       &row, &column)) || (! e2_option_bool_get ("windows-right-click-extra")))
				if (!clist_row_is_selected (view->clist, row))
					gtk_clist_unselect_all(GTK_CLIST(view->clist));
			
			focus_row(view, row, FALSE, FALSE, TRUE);
		}
		
		gint type = 0;
		if (event->state & GDK_SHIFT_MASK) type = 1;
		else if (event->state & GDK_CONTROL_MASK) type = 2;
		e2_menu_show (event->button, event->time, type);
		return TRUE;
	}
	return FALSE;
}

static void select_row_cb (GtkWidget *clist, gint row, gint col, GdkEvent *event, FileView *view);
static gboolean timeout (FileView *clist)
{
	gulong handler = g_signal_handler_find (clist, G_SIGNAL_MATCH_FUNC, 
		0, 0, NULL, select_row_cb, NULL);
	g_signal_handler_unblock (clist, handler);
	return FALSE;
}

//FIXME: this whole function is just a big FIXME, the clist should really be removed
static void select_row_cb (GtkWidget *clist, gint row, gint col, GdkEvent *event, FileView *view)
{
	view->row = row;
//	if (curr_view != view)
//		e2_panes_switch ();
	if (event == NULL)
		return;
	if (event && (event->type == GDK_2BUTTON_PRESS) &&
		!((((GdkEventButton *) event)->state & GDK_SHIFT_MASK) ||
			(((GdkEventButton *) event)->state & GDK_CONTROL_MASK)))
	{
		if ((get_first_selected(curr_view)) == NULL)
			return;
		gulong handler = g_signal_handler_find (clist, G_SIGNAL_MATCH_FUNC, 
			0, 0, NULL, select_row_cb, NULL);
		g_signal_handler_block (clist, handler);
			open_cb(NULL);
		g_timeout_add (100, (void *) timeout, clist);
	}
}

static gint find_filename_begining_with(gchar ch)
{
	gint i;

	/* search from the current row to the end of the list */
	for (i = curr_view->row+1; i < GTK_CLIST(curr_view->clist)->rows; i++)
	{
		FileInfo *info = gtk_clist_get_row_data(GTK_CLIST(curr_view->clist), i);
		
		if (info->filename[0] == ch)
		return i;
	}
	/* search from the first row to the current row */
	for (i = 0; i < curr_view->row; i++)
	{
		FileInfo *info = gtk_clist_get_row_data(GTK_CLIST(curr_view->clist), i);

		if (info->filename[0] == ch)
		return i;
	}
	return -1;
}

static gboolean file_list_key_press_cb(GtkWidget *widget, GdkEventKey *event, FileView *view)
{
	/*
	GList *tmp;

	// get rid of all the modifiers that we don't care about
	event->state &= ~(GDK_LOCK_MASK | GDK_MOD2_MASK | GDK_MOD3_MASK |
		GDK_MOD4_MASK | GDK_MOD5_MASK);
	for (tmp = cfg.key_bindings; tmp != NULL; tmp = tmp->next)
	{
		KeyBinding *kb = tmp->data;

		//key bindings are case insensitive
		if ((event->state == kb->state) &&
			(gdk_keyval_to_lower (event->keyval) == kb->keyval))
		{
			e2_command_run (kb->action, E2_COMMAND_RANGE_DEFAULT);
			return TRUE;
		}
	} */
	if ((event->keyval < 0x100) && (event->state <= 1))
	{ // The key is a letter and the only possible modifier is Shift
		if (e2_option_bool_get ("use-vi-keys"))
		{
		switch (event->keyval)
		{
		case 'j':
		event->keyval = GDK_Down;
		return TRUE;
		case 'k':
		event->keyval = GDK_Up;
		return TRUE;
		case 'l':
		open_cb(NULL);
		gtk_widget_grab_focus(curr_view->clist);
		return TRUE;
		case 'h':
		e2_pane_change_dir (NULL, "..");
		return TRUE;
		case 'g':
		focus_row(view, 0, TRUE, TRUE, TRUE);
		return TRUE;
		case 'G':
		focus_row(view, (GTK_CLIST(view->clist)->rows - 1), TRUE, TRUE, TRUE);
		return TRUE;
		case 't':
		toggle_tag_cb();
		return TRUE;
		default:
		break;
		}
		}
		else
		{
		//Select row beggining with letter entered
		gint row = find_filename_begining_with (event->keyval);
		if (row > 0)
		focus_row(view, row, TRUE, TRUE, TRUE);
		return TRUE;
		}
	}
	return FALSE;
}

static gboolean column_button_cb (GtkWidget *widget, gint col, FileView *view)
{
	printd (DEBUG, "callback: column button");
	GtkSortType direction;

	/* reverse the sorting direction if the list is already sorted by this col */
	if (GTK_CLIST(view->clist)->sort_column == col)
		direction = (GTK_CLIST(view->clist)->sort_type == GTK_SORT_ASCENDING
		 ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING);
	else
		direction = GTK_CLIST(view->clist)->sort_type;
		
	sort_list(view, all_columns[col].sort_func, direction, col);
	return TRUE;
}
/*
 * D&D stuff
 */

/* This gets called when an option from the popup menu (Copy/Move/Link) is
 * chosen
 */
static gboolean drag_op_cb (GtkWidget *widget, gpointer data)
{
	printd (DEBUG, "callback: drag op");
	gchar *dest_dir, *file_list;
	gchar dest[PATH_MAX+NAME_MAX];
	gchar *s, *filename;
	int (*op_func)(gchar *, gchar *) = data;

	gboolean check = e2_option_bool_get ("confirm-overwrite");
	gint result;

	dest_dir = gtk_object_get_data(GTK_OBJECT(widget), "dest_dir");
	file_list = gtk_object_get_data(GTK_OBJECT(widget), "file_list");

	/* file_list is a string of the format:
	 * file:/path/to/one_file\r\nfile:/path/to/second_file\r\n 
	 * and so on
	 */
	while ((s = strstr(file_list, "\r\n")) != NULL)
	{
		*s = '\0';
		file_list += 5; /* Skip over 'file:' */

		filename = strrchr(file_list, '/'); /* extract the filename */
		g_snprintf(dest, sizeof(dest), "%s%s", dest_dir, filename+1);
		if (check && (access(dest, F_OK) == 0))
		{
		gtk_widget_set_sensitive(app.main_window, FALSE);
		result = op_with_ow_check(file_list, dest, op_func);
		gtk_widget_set_sensitive(app.main_window, TRUE);
		if (result == YES_TO_ALL)
		check = FALSE;
		else if (result == CANCEL)
		break;
		}
		else
		op_func(file_list, dest);

		file_list = s + 2; /* Skip over the '\r\n' */
	}
		
	gtk_widget_destroy(app.drag_op_menu);
	refresh_list(other_view);
	refresh_list(other_view);
	return TRUE;
}

/* This is called by the drop handler function clist_drag_data_received */
static void
create_drag_op_menu(gchar *dest_dir, gchar *file_list)
{
	GtkWidget *item;

	if (app.drag_op_menu != NULL)
		gtk_widget_destroy(app.drag_op_menu);

	app.drag_op_menu = gtk_menu_new();
	item = add_menu_item(app.drag_op_menu, _("Copy"), G_CALLBACK (drag_op_cb), file_copy);
	gtk_object_set_data_full(GTK_OBJECT(item), "dest_dir", g_strdup(dest_dir),
		 g_free);
	gtk_object_set_data_full(GTK_OBJECT(item), "file_list", g_strdup(file_list),
		 g_free);
	item = add_menu_item(app.drag_op_menu, _("Move"), G_CALLBACK (drag_op_cb), file_move);
	gtk_object_set_data_full(GTK_OBJECT(item), "dest_dir", g_strdup(dest_dir),
		 g_free);
	gtk_object_set_data_full(GTK_OBJECT(item), "file_list", g_strdup(file_list),
		 g_free);
	item = add_menu_item(app.drag_op_menu, _("SymLink"), G_CALLBACK (drag_op_cb), file_symlink);
	gtk_object_set_data_full(GTK_OBJECT(item), "dest_dir", g_strdup(dest_dir),
		 g_free);
	gtk_object_set_data_full(GTK_OBJECT(item), "file_list", g_strdup(file_list),
		 g_free);

	gtk_menu_popup(GTK_MENU(app.drag_op_menu), NULL, NULL, NULL, NULL, 0, 0);
}

/* This is called automatically when we recieve a drop */
static void
clist_drag_data_received(GtkWidget        *widget,
		 GdkDragContext   *context,
		 gint             x,
		 gint             y,
		 GtkSelectionData *data,
		 guint            drag_info,
		 guint            time,
		 FileView         *view)
{
	printd (DEBUG, "callback: drag data revceived");
	gint row, col;
	FileInfo *info;
	gchar dest_dir[PATH_MAX];

	gtk_clist_set_background(GTK_CLIST(view->clist), view->last_row,
		 &CLIST_COLOR);

	//FIXME: we should get the real pointer height instead of using 25
	if (!gtk_clist_get_selection_info(GTK_CLIST(view->clist), x, y - 25, &row, &col))
	{
		g_snprintf(dest_dir, sizeof(dest_dir), "%s/", view->dir);
	}
	else
	{
		info = gtk_clist_get_row_data(GTK_CLIST(view->clist), row);

		if (is_dir(info))
		g_snprintf(dest_dir, sizeof(dest_dir), "%s/%s/", view->dir, info->filename);
		else
		g_snprintf(dest_dir, sizeof(dest_dir), "%s/", view->dir);
	}

	create_drag_op_menu(dest_dir, (gchar *)data->data);
	gtk_drag_finish(context, FALSE, FALSE, time);
}

static gboolean clist_drag_begin (GtkWidget *widget, GdkDragContext *dc, gpointer data)
{
	printd (DEBUG, "drag begin");
	if (button2updir)
		updir_started = FALSE;
	return TRUE;
}

/* This is called automatically when a drag is initiated 
 * It build a URI list of the selected filenames to set as the drag data
 */
static void
clist_drag_data_get(GtkWidget            *widget,
		GdkDragContext       *context,
		GtkSelectionData     *data,
		guint                info_arg,
		guint                time,
		FileView             *view)
{
	printd (DEBUG, "callback: drag data get");
	FileInfo *info;
	GList *base, *tmp;
	GString *uri_list = g_string_sized_new(PATH_MAX);

	disable_refresh();
	base = tmp = get_selection(view);
	
	for (; tmp != NULL; tmp = tmp->next)
	{
		info = tmp->data;
		g_string_append_printf(uri_list, "file:%s/%s\r\n", view->dir, info->filename);
	}

	gtk_selection_data_set(data, data->target, 8, uri_list->str, uri_list->len);
	g_string_free(uri_list, TRUE);
	g_list_free(base);
	reenable_refresh();
}

/* This handles the highlighting of the rows */
static gboolean clist_drag_motion (GtkWidget *widget,
	GdkDragContext *context, gint x, gint y, guint time, FileView *view)
{
	printd (DEBUG, "callback: drag motion");
	gint row, col;

	//FIXME: this is too slow
//	gint xmouse, ymouse;
//	gdk_window_get_pointer(GTK_CLIST(view->clist)->clist_window,
//		&xmouse, &ymouse, NULL);
	//FIXME: we should get the real pointer height instead of using 25
	if (gtk_clist_get_selection_info (GTK_CLIST(view->clist), x, y - 25, &row, &col))
	{
		if (row != view->last_row)
		{
			gtk_clist_set_background (GTK_CLIST(view->clist), view->last_row,
				&CLIST_COLOR);
			gtk_clist_set_background(GTK_CLIST(view->clist), row, e2_option_color_get ("color-drag-highlight"));
			view->last_row = row;
		}
	} else
		gtk_clist_set_background (GTK_CLIST(view->clist), view->last_row,
			&CLIST_COLOR);

	return TRUE;  
}

static gboolean clist_drag_leave (GtkWidget *widget, GdkDragContext *context,
	guint time, FileView *view)
{
	gtk_clist_set_background(GTK_CLIST(view->clist), view->last_row,
		 &CLIST_COLOR);
	return TRUE;
}

GtkWidget *create_file_view (FileView *view)
{
	printd (DEBUG, "create_file_view (view:)");
	button2updir = e2_option_bool_get ("button2-updir");
	GtkWidget *vbox, *menu_bar, *sw;
	gint i;
	view->clist = NULL;

	// Top Pane 
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox);

	menu_bar = create_filelist_menu_bar(view);
	gtk_widget_show(menu_bar);
	gtk_box_pack_start(GTK_BOX(vbox), menu_bar, FALSE, FALSE, 0);
	
	sw = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_placement(GTK_SCROLLED_WINDOW(sw),
		e2_option_int_get ("scrollbar-position"));
	gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
	gtk_widget_show(sw);

	/* File List */
	view->clist = gtk_clist_new(MAX_COLUMNS);
	for (i = 0; i < MAX_COLUMNS; i++)
	{
		GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
		GtkWidget *label = gtk_label_new(all_columns[i].title);

		view->sort_arrows[i] = gtk_arrow_new(GTK_ARROW_DOWN, GTK_SHADOW_IN);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
		gtk_box_pack_end(GTK_BOX(hbox), view->sort_arrows[i], FALSE, TRUE, 0);

		gtk_widget_show(label);
		gtk_widget_show(hbox);
		gtk_clist_set_column_widget(GTK_CLIST(view->clist), i, hbox);
		GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(view->clist)->column[i].button,
		 GTK_CAN_FOCUS);
	} 

	gtk_clist_column_titles_show(GTK_CLIST(view->clist));
	gtk_clist_set_shadow_type(GTK_CLIST(view->clist), GTK_SHADOW_ETCHED_IN);
	gtk_clist_set_selection_mode(GTK_CLIST(view->clist), GTK_SELECTION_EXTENDED);
	gtk_clist_set_use_drag_icons(GTK_CLIST(view->clist), TRUE);
//	gtk_clist_set_row_height(GTK_CLIST(view->clist), 0);
	
	//set font
	e2_widget_set_font(view->clist, e2_option_str_get ("list-font"),
				 NULL, NULL);

	for (i = 0; i < MAX_COLUMNS; i++)
	{
		gtk_clist_set_column_width(GTK_CLIST(view->clist), i, all_columns[i].size);
		gtk_clist_set_column_visibility(GTK_CLIST(view->clist), i,
		        all_columns[i].is_visible);
	}

	if (button2updir)
		gtk_drag_source_set(view->clist, GDK_BUTTON2_MASK, target_table, n_targets,
			GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
	gtk_drag_dest_set(view->clist, GTK_DEST_DEFAULT_MOTION |
		GTK_DEST_DEFAULT_DROP, target_table, n_targets,
		GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
	g_signal_connect (GTK_OBJECT(view->clist), "button_press_event",
		 G_CALLBACK(mouse_click_cb), view);
	g_signal_connect (GTK_OBJECT(view->clist), "button_release_event",
		 G_CALLBACK(mouse_release_cb), view);
	g_signal_connect(GTK_OBJECT(view->clist), "key_press_event",
		 G_CALLBACK(file_list_key_press_cb), view);
	g_signal_connect(GTK_OBJECT(view->clist), "click_column",
		 G_CALLBACK(column_button_cb), view);
	g_signal_connect(GTK_OBJECT(view->clist), "drag_begin",
		 G_CALLBACK(clist_drag_begin), view);
	g_signal_connect(GTK_OBJECT(view->clist), "drag_data_get",
		 G_CALLBACK(clist_drag_data_get), view);
	g_signal_connect(GTK_OBJECT(view->clist), "drag_motion",
		 G_CALLBACK(clist_drag_motion), view);
	g_signal_connect(GTK_OBJECT(view->clist), "drag_leave",
		 G_CALLBACK(clist_drag_leave), view);
	g_signal_connect(GTK_OBJECT(view->clist), "drag_data_received",
		 G_CALLBACK(clist_drag_data_received), view);
	g_signal_connect(GTK_OBJECT(view->clist), "select_row",
		 G_CALLBACK(select_row_cb), view);

	gtk_container_add(GTK_CONTAINER(sw), view->clist);
	gtk_widget_show(view->clist);

	/* Set the CLIST_COLOR for resetting from the DRAG_HILIGHT color */
	{
		GtkStyle *style = gtk_rc_get_style (view->clist);
		CLIST_COLOR.red   = style->base[GTK_STATE_NORMAL].red;
		CLIST_COLOR.green = style->base[GTK_STATE_NORMAL].green;
		CLIST_COLOR.blue  = style->base[GTK_STATE_NORMAL].blue; 
	}

	return vbox;
}

