/* $Id: e2_utils.c,v 1.23 2004/07/14 23:20:05 cairo Exp $
 * 
 * Copyright (C) 2003 tooar <tooar@gmx.net>
 * Copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <gtk/gtk.h>
#include "emelfm2.h"
#include "build.h"
#include "debug.h"
#include "e2_utils.h"
#include "e2_option.h"
#include "e2_command.h"

E2_Trio *e2_utils_trio_new ()
{
	E2_Trio *t = g_malloc (sizeof (E2_Trio));
	t->a = NULL;
	t->b = NULL;
	t->c = NULL;
	return t;
}

E2_Quintet *e2_utils_quintet_new ()
{
	E2_Quintet *q = g_malloc (sizeof (E2_Quintet));
	q->a = NULL;
	q->b = NULL;
	q->c = NULL;
	q->d = NULL;
	q->e = NULL;
	return q;
}

gchar *e2_utils_color2str (GdkColor *color)
{
	return g_strdup_printf ("#%.2X%.2X%.2X",
		color->red/256, color->green/256, color->blue/256);
}

gchar *e2_utils_str_replace (gchar *str, gchar *old, gchar *new)
{
	gchar **split = g_strsplit (str, old, -1);
	gchar *join = g_strjoinv (new, split);
	g_strfreev (split);
	return join;
}

gchar *e2_utils_str_stretch (gchar *str)
{
	gint len = strlen (str);
	gchar *streched = g_malloc ((len * 2) + 2);
	gint j = 0;
	streched[j++] = ' ';
	gint i;
	for (i = 0; i < len; i++)
	{
		streched[j++] = str[i];
		streched[j++] = ' ';
	}
	streched[j] = '\0';
	return streched;

}

//allocates a new string in lower case: must be freed
gchar *str_to_lower (gchar *string)
{
	return g_utf8_strdown (string, -1);
}

gint S_ISEXE (mode_t mode)
{
	if ((S_IXUSR & mode) || (S_IXGRP & mode) || (S_IXOTH & mode))
		return 1;
	return 0;
}

/* This executes the file (1) command and searches the output for the 
 * given string.
 * Used in is_text and is_executable
 */
static gboolean grep_file_output (gchar *filename, gchar *string)
{
	FILE *pipe;
	gchar line[MAX_LEN];

	g_snprintf (line, sizeof (line), "file \"%s\"", filename);
	if ((pipe = popen (line, "r")) == NULL)
	{
		fprintf (stderr, _("Unable to open pipe for file command\n"));
		return FALSE;
	}

	while (fgets (line, MAX_LEN, pipe))
	{
		if (strstr (line, string))
		{
			pclose (pipe);
			return TRUE;
		}
	}
	pclose (pipe);
	return FALSE;
}

gboolean is_text (gchar *filename)
{
	return grep_file_output (filename, "text");
}

gboolean is_executable (gpointer inf)
{
	FileInfo *info = inf;
	if (S_ISEXE (info->statbuf.st_mode))
		return grep_file_output (info->filename, "executable");

	return FALSE;
}

void view_file (gchar *filename)
{
	gchar *command = g_strconcat (e2_option_str_get ("command-viewer"), " '",
		filename, "'", NULL);
	e2_command_run (command, E2_COMMAND_RANGE_DEFAULT);
	g_free (command);
}

void show_help_file (gchar *filename)
{
	gchar *file = g_build_filename (DOC_DIR, filename, NULL);
	e2_view_dialog_show (file);
	g_free (file);
}

gchar *get_key_name (gint keyval)
{
	return gdk_keyval_name (gdk_keyval_to_lower (keyval));
}

void copy_entry_to_str (GtkWidget *entry, gchar *str)
{
	strcpy (str, gtk_entry_get_text (GTK_ENTRY (entry)));
}

/* This is the function that replaces %f etc. with their appropriate values
 * The GString returned needs to be freed.
 *
 * If for_each is non-null then %f will be replace by its value (which should
 * be a filename) otherwise, %f will be replaced by all selected filenames.
 */
GString *expand_macros (gchar *text, gchar *for_each)
{
	GString *command_string = g_string_new ("");
	gchar *s, *free_this, *command_copy;
	FileInfo *info;

	disable_refresh ();
	command_copy = g_strdup (text);
	free_this = s = command_copy;
	while ((s = strchr (command_copy, '%')) != NULL)
	{
		*s++ = '\0';
		g_string_append (command_string, command_copy);
	gboolean with_quotes = TRUE;
		command_copy = s+1;
	if (s[0] == '%')
	{
		s++;
		command_copy++;
		with_quotes = FALSE;
	}
		switch (*s)
		{
			case 'f':
			{
				if (for_each != NULL)
				{
					if (with_quotes == TRUE)
						g_string_append_printf (command_string, "\"%s\"", for_each);
					else
						g_string_append_printf (command_string, "%s", for_each);
				}
				else
				{
					GList *base, *tmp;

						base = tmp = get_selection (curr_view);
						if (tmp == NULL)
						{
							status_message (_("No files selected\n"));
							g_free (free_this);
							g_string_free (command_string, TRUE);
							reenable_refresh ();
							return NULL;
						}
						else
						{
							for (; tmp != NULL; tmp = tmp->next)
							{
								info = tmp->data;
								if (with_quotes == TRUE)
									g_string_append_printf (command_string, "\"%s\"%s", info->filename,
										tmp->next != NULL ? " " : "");
								else
									g_string_append_printf (command_string, "%s%s", info->filename,
										tmp->next != NULL ? " " : "");
							}
						}
						g_list_free (base);
					}
				}
				break;
			case 'F':
				{
					GList *base, *tmp;

					base = tmp = get_selection (other_view);
					if (tmp == NULL)
					{
						status_message (_("No files selected in other pane\n"));
						g_free (free_this);
						g_string_free (command_string, TRUE);
						reenable_refresh ();
						return NULL;
					}
					else
					{
						for (; tmp != NULL; tmp = tmp->next)
						{
							info = tmp->data;
							if (with_quotes == TRUE)
								g_string_append_printf (command_string, "\"%s/%s\"%s", other_view->dir, info->filename,
										tmp->next != NULL ? " " : "");
							else
								g_string_append_printf (command_string, "%s/%s%s", other_view->dir, info->filename,
										tmp->next != NULL ? " " : "");
						}
					}
					g_list_free (base);
				}
				break;
			case 'd':
				if (with_quotes == TRUE)
					g_string_append_printf (command_string, "\"%s\"", curr_view->dir);
				else
					g_string_append_printf (command_string, "%s", curr_view->dir);
				break;
			case 'D':
				if (with_quotes == TRUE)
					g_string_append_printf (command_string, "\"%s\"", other_view->dir);
				else
					g_string_append_printf (command_string, "%s", other_view->dir);
				break;
			case '{':
				if ((s = strchr (command_copy, '}')) == NULL)
				{
					status_message (_("No matching '}' found in action text.\n"));
					g_free (free_this);
					g_string_free (command_string, TRUE);
					reenable_refresh ();
					return NULL;
				}
				else
				{
					gchar *user_input;

					*s++ = '\0';
					create_user_prompt (command_copy, "", FALSE, &user_input);
					gtk_main ();
					command_copy = s;
					if (user_input != NULL)
					{
						g_string_append (command_string, user_input);
						g_free (user_input);
					}
					else
					{
						g_free (free_this);
						g_string_free (command_string, TRUE);
						reenable_refresh ();
						return NULL;
					}
				}
				break;
			default:
				g_string_append_c (command_string, '%');
				g_string_append_c (command_string, *s);
				break;
		}
	}
	g_string_append (command_string, command_copy);
	g_free (free_this);
	reenable_refresh ();
	return command_string;
}

//translate transparent link file/dir names
gchar *e2_translate_to_filename (gchar *current_dir, gchar *new_file)
{
	//remove leading and trailing whitespaces
	new_file = g_strchug (g_strchomp (new_file));

	//save if new path was absolute for later usage
	gboolean absolute = g_path_is_absolute (new_file);
	gint new_file_len = strlen (new_file) - 1;

	//is new_file the file system root?
	if (g_str_equal (new_file, G_DIR_SEPARATOR_S))
		return g_strdup (G_DIR_SEPARATOR_S);
	
	while (new_file[new_file_len] == '/') new_file[new_file_len--] = '\0';
	while (new_file[0] == '/') new_file++;
	
	//remove superfluous slashes from within the new path
	gchar clean[new_file_len];
	gint clean_count = 0;
	gint i;
	for (i = 0; i <= new_file_len; i++)
	{
		if (i < new_file_len)
		{
			if (!(new_file[i] == '/' && new_file[i + 1] == '/'))
				clean[clean_count++] = new_file[i];
		} else
			clean[clean_count++] = new_file[i];

	}
	clean[clean_count] = '\0';
	
	//split new file
	gchar **split = g_strsplit (clean, G_DIR_SEPARATOR_S, -1);
	gint split_count = 0;
	while (split[split_count] != NULL) split_count++;

	//split current dir
	gchar **split_old = g_strsplit (current_dir, G_DIR_SEPARATOR_S, -1);
	gint split_old_count = 0;
	while (split_old[split_old_count] != NULL) split_old_count++;
	//remove trailing slash from current dir
	split_old_count--;

	//max number of elements the join array may have
	gint join_max_count = split_old_count + split_count + 2;
	
	//used to save the new path that has to be joined
	gchar *join[join_max_count];
	gint join_count = 0;
	
	//if the new file name is not absolute
	//prepend the current dir
	if (absolute != TRUE)
	{
		for (i = 0; i < split_old_count; i++)
		{
			join[join_count++] = split_old[i];
		}
	//to get the root slash
	} else
		join[join_count++] = "";

	//now add the new file elements
	for (i = 0; i < split_count; i++)
	{
		gint cur_len = strlen (split[i]);
		//we know for sure that the split str len is >= 1
		//referring to current path, ignore
		if ((cur_len == 1) && (split[i][0] == '.'))
		{
		//up dir
		} else
		if (cur_len == 2)
		{
			if (((split[i][0]) == '.') && ((split[i][1]) == '.'))
			{
				if (join_count > 0) join_count--;
 			} else
				join[join_count++] = split[i];
		} else
			join[join_count++] = split[i];
	}

	if (join_count == 0)
		join[join_count++] = "";
	join[join_count++] = "";
	//fill the left (empty) element field with NULL
	for (; join_count < join_max_count; join_count++)
		join[join_count] = NULL;

	//first concat the retval
	gchar *retval = g_strjoinv (G_DIR_SEPARATOR_S, join);

	//clean up
	g_strfreev (split);
	g_strfreev (split_old);

	return retval;
}

/**
 * @brief update gtk properties like menu delays
 *
 * update gtk internal properties. these are currently only menu
 * popup and popdown delays. this function is usually called
 * after configuration changes to update the gtk properties.
 *
 * @return 
 **/
void e2_update_settings ()
{
	GtkSettings *s = gtk_settings_get_default ();
	gint delay_up = e2_option_int_get ("submenu-up-delay");
	gint delay_down = e2_option_int_get ("submenu-down-delay");
	//be on the safe side
	if (delay_up < 0) delay_up = 0;
	if (delay_down < 0) delay_down = 0;
	gtk_settings_set_long_property (s, "gtk-menu-popup-delay",
		delay_up, "XProperty");
	gtk_settings_set_long_property (s, "gtk-menu-popdown-delay",
		delay_down, "XProperty");
	gtk_settings_set_long_property (s, "gtk-menubar-popup-delay",
		delay_up, "XProperty");
//	gtk_rc_parse_string ("style \"e2_default\" { GtkComboBoxEntry::appears-as-list = 1 } class \"*\" style \"e2_default\"");
}

/**
 * @brief replaces environment variables in a string.
 *
 * replaces all references of environment variables in @p raw. the references
 * have to have the form @c \$(VAR) where @c VAR is the name of the environment
 * variable.
 *
 * @code
 * gchar *str = e2_utils_replace_env_var ("my home is \$(HOME) and i am \$(USERNAME)");
 * @endcode
 * 
 * @param raw input string with references to replace
 *
 * @return newly allocated string that has to be freed.
 **/
gchar *e2_utils_replace_env_vars (gchar *raw)
{
//	printd (DEBUG, "e2_utils_replace_env_vars (raw:%s)", raw);
	gchar **split = g_strsplit (raw, "$(", -1);
	gint split_count = 0;
	while (split[split_count] != NULL) split_count++;

	//early exit in case we don't have to replace anything
	if (split_count < 2)
	{
		g_strfreev (split);
		return e2_utils_replace_options (raw);
	}

	gint join_count = 0;
	gchar *join[split_count * 2];

	gint i;
	for (i = 0; i < split_count; i++)
	{
		gchar *rest = strchr (split[i], ')');
		if (rest != NULL)
		{
			*rest++ = '\0';
			const gchar *env = g_getenv (split[i]);
			if (env != NULL)
				join[join_count++] = (gchar *) env;
			join[join_count++] = rest;
		}
		else
		{
			if (i > 0)
				join[join_count++] = "$(";
			else if (*split == '\0')
				join[join_count++] = "$(";
			join[join_count++] = split[i];
		}
	}
	join[join_count] = NULL;

	gchar *retval = g_strjoinv (NULL, join);

	g_strfreev (split);
	gchar *retval_real = e2_utils_replace_options (retval);
	g_free (retval);
	return retval_real;
}

gchar *e2_utils_replace_options (gchar *raw)
{
//	printd (DEBUG, "e2_utils_replace_options (raw:%s)", raw);
	gchar **split = g_strsplit (raw, "$[", -1);
	gint split_count = 0;
	while (split[split_count] != NULL) split_count++;

	//early exit in case we don't have to replace anything
	if (split_count < 2)
	{
		g_strfreev (split);
		return g_strdup (raw);
	}

	gint join_count = 0;
	gchar *join[split_count * 2];

	gint i;
	for (i = 0; i < split_count; i++)
	{
		gchar *rest = strchr (split[i], ']');
		if (rest != NULL)
		{
			*rest++ = '\0';
			E2_OptionSet *set = e2_option_get (split[i]);
			if (set != NULL)
				join[join_count++] = _e2_option_str_get (set);
			join[join_count++] = rest;
		}
		else
		{
			if (i > 0)
				join[join_count++] = "$(";
			else if (*split == '\0')
				join[join_count++] = "$(";
			join[join_count++] = split[i];
		}
	}
	join[join_count] = NULL;

	gchar *retval = g_strjoinv (NULL, join);

	g_strfreev (split);
	return retval;
}

/* create relative path */
void e2_utils_rel_path (gchar* src, gchar* dest, gchar* rel)
{
	gchar* tsp = src;

	/* strip common paths */
	while (*src && (*src == *dest))
	{
		*dest++;
		if ('/' == *src++)
			/* save as short path */
			tsp = src;
	}
	/* insert short dest dirs (if any) as uplinks */
	while (*dest)
	{
		if (*dest++ == '/')
		{
			*rel++ = '.';
			*rel++ = '.';
			*rel++ = '/';
		}
	}
	/* and add short src path */
	strcpy (rel, tsp);
}

void e2_utils_get_abs_pos (GtkWidget *widget, gint *x, gint *y)
{
	if (GTK_WIDGET_TOPLEVEL (widget))
		gdk_window_get_position (widget->window, x, y);
	else
	{
		GdkWindow *window;

		if (GTK_WIDGET_NO_WINDOW (widget))
			window = widget->window;
		else
			window = gdk_window_get_parent (widget->window);

		gdk_window_get_origin (window, x, y);

		*x += widget->allocation.x;
		*y += widget->allocation.y;
	}
}
